/*
 * AutoRaster.hpp
 *
 * Raster class wrapped to be pointer-copyable with
 * ownership passing (auto_pointer-like).
 *
 */
#ifndef INCLUDE_GRAPHICS_AUTORASTER_HPP_FILE
#define INCLUDE_GRAPHICS_AUTORASTER_HPP_FILE

/* public header */

#include <memory>

#include "Graphics/Raster.hpp"


namespace Graphics
{

template<typename TPixel>
class AutoRaster: public std::auto_ptr< Raster<TPixel> >
{
public:
  typedef Raster<TPixel> TRaster;

  // create image of a given size
  explicit AutoRaster(const Graphics::Size &size):
    std::auto_ptr<TRaster>( new TRaster(size) )
  {
  }
  // initialize from pointer
  explicit AutoRaster(TRaster *r):
    std::auto_ptr<TRaster>(r)
  {
  }

  // this solution is not very butiful, but this is
  // how auto_ptr works indeed and this has to be copied
  // in order to have the same functionality...
  AutoRaster(std::auto_ptr<TRaster> aptr):
    std::auto_ptr<TRaster>(aptr)
  {
  }
  AutoRaster &operator=(std::auto_ptr<TRaster> aptr)
  {
    reset( aptr.release() );
    return *this;
  }
  operator std::auto_ptr<TRaster>& (void)
  {
    return *this;
  }
}; // class AutoRaster

//
// here are some commonly used definitions
//
typedef AutoRaster<PixelRGB>    AutoRasterRGB;
typedef AutoRaster<PixelMono>   AutoRasterMono;
typedef AutoRaster<PixelBinary> AutoRasterBinary;

} // namespace Graphics

#endif

