/*
 * CenterPass.t.cpp
 *
 */
#include <tut.h>

#include "Graphics/Filter/CenterPass.hpp"

namespace Graphics
{
namespace Filter
{

struct CenterPassTestClass
{
  CenterPassTestClass():
    _size( Size(2,3) ),
    _rIn (_size),
    _rOut(_size)
  {
    _rIn.setPixel( Position(0,0), PixelMono( 10) );
    _rIn.setPixel( Position(0,1), PixelMono(255) );
    _rIn.setPixel( Position(0,2), PixelMono(210) );
    _rIn.setPixel( Position(1,0), PixelMono( 90) );
    _rIn.setPixel( Position(1,1), PixelMono(110) );
    _rIn.setPixel( Position(1,2), PixelMono( 50) );

    // fill with some unusual value
    _rOut.fill( PixelMono(99) );
  }

  const Size _size;
  RasterMono _rIn;
  RasterMono _rOut;
};

} // namespace Filter
} // namespace Graphics


namespace tut
{
typedef Graphics::Filter::CenterPassTestClass TestClass;
typedef test_group<TestClass> factory;
typedef factory::object testObj;
} // namespace tut


namespace
{
tut::factory tf("Graphics/Filter/CenterPass");
}


using namespace Graphics;
using namespace Graphics::Filter;

namespace tut
{

// test if transformation is const, if required
template<>
template<>
void testObj::test<1>(void)
{
  CenterPass th(0,255);
  th.filter(_rOut, _rIn);

  // check
  for(unsigned int y=0; y<_rIn.size().y(); ++y)
    for(unsigned int x=0; x<_rIn.size().x(); ++x)
    {
      const Position pos(x,y);
      ensure( _rOut.getPixel(pos).get() == _rIn.getPixel(pos).get() );
    }
}

// test upper boundary
template<>
template<>
void testObj::test<2>(void)
{
  CenterPass th(0,110, 0,199);
  th.filter(_rOut, _rIn);

  ensure( _rOut.getPixel( Position(0,0) ).get()== 10 );
  ensure( _rOut.getPixel( Position(0,1) ).get()==199 );
  ensure( _rOut.getPixel( Position(0,2) ).get()==199 );
  ensure( _rOut.getPixel( Position(1,0) ).get()== 90 );
  ensure( _rOut.getPixel( Position(1,1) ).get()==110 );
  ensure( _rOut.getPixel( Position(1,2) ).get()== 50 );
}

// test lower bound
template<>
template<>
void testObj::test<3>(void)
{
  CenterPass th(110,255, 9,0);
  th.filter(_rOut, _rIn);

  ensure( _rOut.getPixel( Position(0,0) ).get()==  9 );
  ensure( _rOut.getPixel( Position(0,1) ).get()==255 );
  ensure( _rOut.getPixel( Position(0,2) ).get()==210 );
  ensure( _rOut.getPixel( Position(1,0) ).get()==  9 );
  ensure( _rOut.getPixel( Position(1,1) ).get()==110 );
  ensure( _rOut.getPixel( Position(1,2) ).get()==  9 );
}


// test if thresholding will throw on bad image sizes value
template<>
template<>
void testObj::test<4>(void)
{
  CenterPass th(100,255);
  try
  {
    th.filter(_rOut, RasterMono( Size(12,13) ) );    // should throw
    fail("threshold() didn't throw on input/output images sizes mismatch");
  }
  catch(const Exception &ex)
  {
    // this is expected behaviour
  }
}


// test if one-arg version of thresholding works as well
template<>
template<>
void testObj::test<5>(void)
{
  CenterPass th(110,255, 9,0);

  th.filter(_rOut, _rIn);
  AutoRasterMono ar=th.filterNew(_rIn);

  // check
  for(unsigned int y=0; y<_rIn.size().y(); ++y)
    for(unsigned int x=0; x<_rIn.size().x(); ++x)
    {
      const Position pos(x,y);
      ensure( _rOut.getPixel(pos).get() == ar->getPixel(pos).get() );
    }
}

} // namespace tut

