/*
 * ColorConverter.hpp
 *
 * transforation for changing given color to
 * another, specified by external function object.
 *
 */
#ifndef INCLUDE_GRAPHICS_FILTER_COLORCONVERTER_HPP_FILE
#define INCLUDE_GRAPHICS_FILTER_COLORCONVERTER_HPP_FILE

/* public header */

#include "Graphics/Raster.hpp"
#include "Graphics/PixelTransform/PixelConverter.hpp"
#include "Graphics/Filter/GeneralRaster.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace Filter
{

// this wrapper uses PixelConverter to operate on pixels
template<typename TPixOut, typename TPixIn>
class ColorConverterProcessor
{
public:
  inline TPixOut operator()(const Position       &pos,
                            const Raster<TPixIn> &in  ) const
  {
    const TPixIn &pix=in.getPixel(pos);
    return _pc.operator()<TPixOut,TPixIn>(pix);
  }

private:
  Graphics::PixelTransform::PixelConverter _pc;
}; // class ColorConverterProcessor


//
// template for converting classes
template<typename TPixOut, typename TPixIn>
class ColorConverter:
        public GeneralRasterAny<TPixOut, TPixIn,
                                ColorConverterProcessor<TPixOut, TPixIn> >
{
public:
  typedef ColorConverterProcessor<TPixOut, TPixIn>     TConvProc;
  typedef GeneralRasterAny<TPixOut, TPixIn, TConvProc> TBase;

  ColorConverter(void):
    TBase( TConvProc() )
  {
  }
}; // class ColorConverter

} // namespace Filter
} // namespace Graphics

#endif

