/*
 * CoolorConverter.t.cpp
 *
 */
#include <tut.h>

#include "Graphics/Filter/ColorConverter.hpp"


namespace Graphics
{
namespace Filter
{

struct ColorConverterTestClass
{
  ColorConverterTestClass(void):
    _size(1,2),
    _in (_size),
    _out(_size)
  {
  }

  const Size _size;
  RasterMono _in;
  RasterMono _out;
};

} // namespace Filter
} // namespace Graphics



namespace tut
{
typedef Graphics::Filter::ColorConverterTestClass TestClass;
typedef test_group<TestClass> factory;
typedef factory::object testObj;
} // namespace tut


namespace
{
tut::factory tf("Graphics/Filter/ColorConverter");
}


using namespace Graphics;
using namespace Graphics::PixelTransform;
using namespace Graphics::Filter;


namespace tut
{

// test if it works in regular conditions
template<>
template<>
void testObj::test<1>(void)
{
  {
    const Position pos(0,0);
    _in.setPixel( pos, PixelMono(180) );
  }
  {
    const Position pos(0,1);
    _in.setPixel( pos, PixelMono( 90) );
  }

  ColorConverter<PixelBinary, PixelMono> cc;
  AutoRaster<PixelBinary>                ar=cc.filterNew(_in); // transform image
  PixelConverter                         pc;
  // check if results matches
  {
    const Position pos(0,0);
    ensure( ar->getPixel(pos).get()==
            pc.operator()<PixelBinary,PixelMono>( _in.getPixel(pos) ).get() );
  }
  {
    const Position pos(0,1);
    ensure( ar->getPixel(pos).get()==
            pc.operator()<PixelBinary,PixelMono>( _in.getPixel(pos) ).get() );
  }
}

// check for throw on size mismatch
template<>
template<>
void testObj::test<2>(void)
{
  RasterRGB rc( Size(2,9) );
  try
  {
    ColorConverter<PixelRGB, PixelMono> cc;
    cc.filter(rc, _in);
    fail("comparing output image size with input didn't failed");
  }
  catch(const Graphics::Exception &ex)
  {
    // this is expected behavior
  }
}

} // namespace tut

