/*
 * ColorConverterTransformation.hpp
 *
 */
#ifndef INCLUDE_GRAPHICS_FILTER_COLORCONVERTERTRANSFORMATION_HPP_FILE
#define INCLUDE_GRAPHICS_FILTER_COLORCONVERTERTRANSFORMATION_HPP_FILE

/* public header */

#include <boost/shared_ptr.hpp>

#include "Graphics/SharedRaster.hpp"
#include "Graphics/Filter/ColorConverter.hpp"
#include "Graphics/Transform/Transformation.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace Filter
{

template<typename TPixOut, typename TPixIn>
class ColorConverterTransformation: public Graphics::Transform::Transformation
{
public:
  typedef SharedRaster<TPixIn>  TShrRasterIn;
  typedef SharedRaster<TPixOut> TShrRasterOut;

  ColorConverterTransformation(TShrRasterOut rOut, TShrRasterIn rIn):
    _rIn (rIn),
    _rOut(rOut)
  {
  }

  inline TShrRasterIn getRasterIn(void) const
  {
    return _rIn;
  }
  inline TShrRasterOut getRasterOut(void) const
  {
    return _rOut;
  }

  virtual void transform(void)
  {
    _cc.filter( *_rOut.get(), *_rIn.get() );
  }

private:
  ColorConverter<TPixOut, TPixIn> _cc;
  TShrRasterIn                    _rIn;
  TShrRasterOut                   _rOut;
}; // class ColorConverterTransformation

} // namespace Filter
} // namespace Graphics

#endif

