/*
 * GeneralLayer.t.cpp
 *
 */
#include <tut.h>

#include "Graphics/Filter/GeneralLayer.hpp"


namespace
{
struct SomeTestOperation
{
  Graphics::color operator()(const Graphics::Position        &pos,
                             const Graphics::RasterLayerMono &in) const
  {
    const Graphics::color &pIn=in.getPixel(pos);
    return (pIn<255)?(pIn+1):255;
  }
}; // struct SomeTestOperation

struct SomeTestOperation2
{
  Graphics::color operator()(const Graphics::Position        &pos,
                             const Graphics::RasterLayerMono &in) const
  {
    return in.getPixel(pos)+2;
  }
}; // struct SomeTestOperation2

} // unnamed namespace


namespace Graphics
{
namespace Filter
{

struct GeneralLayerTestClass
{
  GeneralLayerTestClass(void):
    _size(4,3),
    _in  (_size),
    _inl ( makeRasterLayerMono(&_in) ),
    _out (_size),
    _outl( makeRasterLayerMono(&_out) ),
    _gf  (_sto)
  {
  }

  const Size        _size;
  RasterMono        _in;
  RasterLayerMono   _inl;
  RasterMono        _out;
  RasterLayerMono   _outl;
  SomeTestOperation _sto;
  GeneralLayer<PixelMono, SomeTestOperation> _gf;
};

} // namespace Filter
} // namespace Graphics


namespace tut
{
typedef Graphics::Filter::GeneralLayerTestClass TestClass;
typedef test_group<TestClass> factory;
typedef factory::object testObj;
} // namespace tut


namespace
{
tut::factory tf("Graphics/Filter/GeneralLayer");
}


using namespace Graphics;
using namespace Graphics::Filter;

namespace tut
{

// test if it works in regular conditions
template<>
template<>
void testObj::test<1>(void)
{
  // fransform manualy
  for(unsigned int y=0; y<_in.size().y(); ++y)
    for(unsigned int x=0; x<_in.size().x(); ++x)
    {
      const Position pos(x,y);
      _outl.setPixel( pos, _sto(pos, _inl) );
    }

  // make automatic transform
  RasterMono      rm( _in.size() );
  RasterLayerMono rlm=makeRasterLayerMono(&rm);
  _gf.filter(rlm, _inl);

  // compare automatic and manual results
  for(unsigned int y=0; y<rm.size().y(); ++y)
    for(unsigned int x=0; x<rm.size().x(); ++x)
    {
      const Position   pos(x,y);
      const PixelMono &p1=rm.getPixel(pos);
      const PixelMono &p2=_out.getPixel(pos);
      ensure(p1.get()==p2.get());
    }
}

// check for throw on size mismatch
template<>
template<>
void testObj::test<2>(void)
{
  RasterMono      rg( Size(2,9) );
  RasterLayerMono rlg=makeRasterLayerMono(&rg);
  try
  {
    _gf.filter(rlg, _inl);
    fail("comparing output image size with input didn't failed");
  }
  catch(const Graphics::Exception &ex)
  {
    // this is expected behavior
  }
}

// enshure that template arguments sequence is correct
template<>
template<>
void testObj::test<3>(void)
{
  // input
  RasterMono      rg( Size(1,1) );
  RasterLayerMono rlg=makeRasterLayerMono(&rg);
  // output
  RasterRGB       rc( Size(1,1) );
  RasterLayerRGB  rlc=makeRasterLayerRGB(&rc, PixelRGB::Layer::R);

  const color pm=120;
  rlg.fill(pm);
  GeneralLayerAny<PixelRGB, PixelMono, SomeTestOperation2>
    ga( (SomeTestOperation2()) );

  const PixelRGB pPrev=rc.getPixel( Position(0,0) );
  ga.filter(rlc, rlg);
  const PixelRGB p=rc.getPixel( Position(0,0) );
  ensure( p.getR()==pm+2 &&
          p.getG()==pPrev.getG() &&
          p.getB()==pPrev.getB()    );
}

} // namespace tut

