/*
 * GeneralLayerTransformation.hpp
 *
 * transformation of image filtering
 *
 */
#ifndef INCLUDE_GRAPHICS_FILTER_GENERALLAYERTRANSFORMATION_HPP_FILE
#define INCLUDE_GRAPHICS_FILTER_GENERALLAYERTRANSFORMATION_HPP_FILE

/* public header */

#include "Graphics/Filter/GeneralLayer.hpp"
#include "Graphics/Transform/Transformation.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace Filter
{

template<typename TPixOut, typename TPixIn, typename TPixOp>
class GeneralLayerAnyTransformation: public
                    Graphics::Transform::Transformation
{
public:
  typedef RasterLayer<TPixIn>  TRasterLayerIn;
  typedef RasterLayer<TPixOut> TRasterLayerOut;

  GeneralLayerAnyTransformation(const TPixOp    &pixOp,
                                TRasterLayerOut  rOut,
                                TRasterLayerIn   rIn):
    _gf  (pixOp),
    _rIn (rIn),
    _rOut(rOut)
  {
  }

  inline TRasterLayerIn  getRasterIn(void) const
  {
    return _rIn;
  }
  inline TRasterLayerOut getRasterOut(void) const
  {
    return _rOut;
  }

  virtual void transform(void)
  {
    _gf.filter(_rOut, _rIn);
  }

private:
  GeneralLayerAny<TPixOut, TPixIn, TPixOp> _gf;
  TRasterLayerIn  _rIn;
  TRasterLayerOut _rOut;
}; // class GeneralLayerAnyTransformation



template<typename TPixel, typename TPixOp>
class GeneralLayerTransformation: public
        GeneralLayerAnyTransformation<TPixel, TPixel, TPixOp>
{
public:
  typedef RasterLayer<TPixel> TRasterLayer;
  typedef GeneralLayerAnyTransformation<TPixel, TPixel, TPixOp> TBase;

  GeneralLayerTransformation(TPixOp       pixOp,
                             TRasterLayer rOut,
                             TRasterLayer rIn):
    TBase(pixOp, rOut, rIn)
  {
  }
}; // class GeneralLayerTransformation


} // namespace Filter
} // namespace Graphics

#endif

