/*
 * GeneralLayerTransformation.t.cpp
 *
 */
#include <tut.h>

#include "Graphics/Filter/GeneralLayerTransformation.hpp"

namespace Graphics
{
namespace Filter
{

struct GeneralLayerTransformationTestClass
{
};

} // namespace Filter
} // namespace Graphics


namespace tut
{
typedef Graphics::Filter::GeneralLayerTransformationTestClass TestClass;
typedef test_group<TestClass> factory;
typedef factory::object testObj;
} // namespace tut


namespace
{
tut::factory tf("Graphics/Filter/GeneralLayerTransformation");
}


using namespace boost;
using namespace Graphics;
using namespace Graphics::Filter;

namespace
{
struct SomeTestOperation
{
  Graphics::color operator()(const Graphics::Position        &pos,
                             const Graphics::RasterLayerMono &in) const
  {
    const Graphics::color &pIn=in.getPixel(pos);
    return (pIn<255)?(pIn+1):255;
  }
}; // struct SomeTestOperation
struct SomeTestOperationC
{
  Graphics::color operator()(const Graphics::Position       &pos,
                             const Graphics::RasterLayerRGB &in) const
  {
    const Graphics::color &pIn=in.getPixel(pos);
    return (pIn<255)?(pIn+1):255;
  }
}; // struct SomeTestOperationC
} // unnamed namespace



namespace tut
{

// test filtering via Transformation and check if it matches
template<>
template<>
void testObj::test<1>(void)
{
  // some random input raster
  const Size      size(10,20);

  RasterMono      rgIn(size);
  RasterLayerMono rgInl=makeRasterLayerMono(&rgIn);

  RasterMono      rgOutT(size);
  RasterLayerMono rgOutTl=makeRasterLayerMono(&rgOutT);

  // output raster after 'regular' filtering
  RasterMono      rgOutR(size);
  RasterLayerMono rgOutRl=makeRasterLayerMono(&rgOutR);

  SomeTestOperation sto;
  GeneralLayerTransformation<PixelMono, SomeTestOperation>
    gft(sto, rgOutTl, rgInl);
  gft.transform();
  GeneralLayer<PixelMono, SomeTestOperation> grTmp(sto);
  grTmp.filter(rgOutRl, rgInl);

  for(unsigned int y=0; y<rgOutT.size().y(); ++y)
    for(unsigned int x=0; x<rgOutT.size().x(); ++x)
    {
      const Position  pos(x,y);
      const PixelMono p1=rgOutT.getPixel(pos);
      const PixelMono p2=rgOutR.getPixel(pos);
      ensure(p1.get()==p2.get());
    }
}

// try building AnyTransform and see if it's ok.
template<>
template<>
void testObj::test<2>(void)
{
  // some random input raster
  const Size      size(10,20);

  RasterRGB       rgIn(size);
  RasterLayerRGB  rgInl=makeRasterLayerRGB(&rgIn, PixelRGB::Layer::R);

  RasterMono      rgOutT(size);
  RasterLayerMono rgOutTl=makeRasterLayerMono(&rgOutT);

  // output raster after 'regular' filtering
  RasterMono      rgOutR(size);
  RasterLayerMono rgOutRl=makeRasterLayerMono(&rgOutR);

  SomeTestOperationC stoc;
  GeneralLayerAnyTransformation<PixelMono, PixelRGB, SomeTestOperationC>
    gft(stoc, rgOutTl, rgInl);
  gft.transform();
  GeneralLayerAny<PixelMono, PixelRGB, SomeTestOperationC> grTmp(stoc);
  grTmp.filter(rgOutRl, rgInl);

  for(unsigned int y=0; y<rgOutT.size().y(); ++y)
    for(unsigned int x=0; x<rgOutT.size().x(); ++x)
    {
      const Position  pos(x,y);
      const PixelMono p1=rgOutT.getPixel(pos);
      const PixelMono p2=rgOutR.getPixel(pos);
      ensure(p1.get()==p2.get());
    }
}

} // namespace tut

