/*
 * GeneralLayerTwo.hpp
 *
 */
#ifndef INCLUDE_GRAPHICS_FILTER_GENERALLAYERTWO_HPP_FILE
#define INCLUDE_GRAPHICS_FILTER_GENERALLAYERTWO_HPP_FILE

/* public header */

#include <boost/noncopyable.hpp>
#include <cassert>

#include "Graphics/Filter/GeneralTwo.hpp"
#include "Graphics/RasterLayer.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace Filter
{

//
// generic class - input and output types may differ
//
template<typename TPixOut, typename TPixIn1,
         typename TPixIn2, typename TPixOp>
class GeneralLayerTwoAny: public GeneralTwoAny< RasterLayer<TPixOut>,
                                                RasterLayer<TPixIn1>,
                                                RasterLayer<TPixIn2>,
                                                TPixOp >
{
public:
  typedef RasterLayer<TPixIn1>     TRasterLayerIn1;
  typedef RasterLayer<TPixIn2>     TRasterLayerIn2;
  typedef RasterLayer<TPixOut>     TRasterLayerOut;

  GeneralLayerTwoAny(const TPixOp &op):
    GeneralTwoAny< TRasterLayerOut, TRasterLayerIn1,
                   TRasterLayerIn2, TPixOp >(op)
  {
  }
}; // class GeneralLayerTwoAny


//
// class version assuming to have the same in1/in2/out
// raster types.
//
template<typename TPixel, typename TPixOp>
class GeneralLayerTwo: public
            GeneralLayerTwoAny<TPixel, TPixel, TPixel, TPixOp>
{
public:
  GeneralLayerTwo(const TPixOp &op):
    GeneralLayerTwoAny<TPixel, TPixel, TPixel, TPixOp>(op)
  {
  }
}; // class GeneralLayerTwo

} // namespace Filter
} // namespace graphics

#endif

