/*
 * GeneralLayerTwoTransformation.hpp
 *
 * transformation of image filtering
 *
 */
#ifndef INCLUDE_GRAPHICS_FILTER_GENERALLAYERTWOTRANSFORMATION_HPP_FILE
#define INCLUDE_GRAPHICS_FILTER_GENERALLAYERTWOTRANSFORMATION_HPP_FILE

/* public header */

#include "Graphics/Filter/GeneralLayerTwo.hpp"
#include "Graphics/Transform/Transformation.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace Filter
{

template<typename TPixel, typename TPixOp>
class GeneralLayerTwoTransformation: public
                Graphics::Transform::Transformation
{
public:
  typedef RasterLayer<TPixel> TRasterLayer;

  GeneralLayerTwoTransformation(const TPixOp &pixOp,
                                TRasterLayer  rOut,
                                TRasterLayer  rIn1,
                                TRasterLayer  rIn2):
    _gf  (pixOp),
    _rIn1(rIn1),
    _rIn2(rIn2),
    _rOut(rOut)
  {
  }

  inline TRasterLayer getRasterLayerIn1(void) const
  {
    return _rIn1;
  }
  inline TRasterLayer getRasterLayerIn2(void) const
  {
    return _rIn2;
  }
  inline TRasterLayer getRasterLayerOut(void) const
  {
    return _rOut;
  }

  virtual void transform(void)
  {
    _gf.filter(_rOut, _rIn1, _rIn2);
  }

private:
  GeneralLayerTwo<TPixel, TPixOp> _gf;
  TRasterLayer                    _rIn1;
  TRasterLayer                    _rIn2;
  TRasterLayer                    _rOut;
}; // class GeneralLayerTwoTransformation

} // namespace Filter
} // namespace Graphics

#endif

