/*
 * GeneralLayerTwoTransformation.t.cpp
 *
 */
#include <tut.h>
#include <algorithm>

#include "Graphics/Filter/GeneralLayerTwoTransformation.hpp"

namespace Graphics
{
namespace Filter
{

struct GeneralLayerTwoTransformationTestClass
{
};

} // namespace Filter
} // namespace Graphics


namespace tut
{
typedef Graphics::Filter::GeneralLayerTwoTransformationTestClass TestClass;
typedef test_group<TestClass> factory;
typedef factory::object testObj;
} // namespace tut


namespace
{
tut::factory tf("Graphics/Filter/GeneralLayerTwoTransformation");
}


using namespace boost;
using namespace Graphics;
using namespace Graphics::Filter;

namespace
{
struct SomeTestOperation
{
  Graphics::color operator()(const Graphics::Position        &pos,
                             const Graphics::RasterLayerMono &in1,
                             const Graphics::RasterLayerMono &in2) const
  {
    const Graphics::color &pIn1=in1.getPixel(pos);
    const Graphics::color &pIn2=in2.getPixel(pos);
    return std::min(pIn1, pIn2);
  }
}; // struct SomeTestOperation
} // unnamed namespace



namespace tut
{

// test filtering via Transformation and check if it matches
template<>
template<>
void testObj::test<1>(void)
{
  // some random input raster
  const Size size(10,20);

  RasterMono      rgIn1 (size);
  RasterLayerMono rgIn1l=makeRasterLayerMono(&rgIn1);

  RasterMono      rgIn2 (size);
  RasterLayerMono rgIn2l=makeRasterLayerMono(&rgIn2);

  RasterMono      rgOutT(size);
  RasterLayerMono rgOutTl=makeRasterLayerMono(&rgOutT);

  // output raster after 'regular' filtering
  RasterMono      rgOutR(size);
  RasterLayerMono rgOutRl=makeRasterLayerMono(&rgOutR);

  // use transformation
  SomeTestOperation sto;
  GeneralLayerTwoTransformation<PixelMono, SomeTestOperation>
    gft(sto, rgOutTl, rgIn1l, rgIn2l);
  gft.transform();

  // transform manualy
  GeneralLayerTwo<PixelMono, SomeTestOperation> grt(sto);
  grt.filter(rgOutRl, rgIn1l, rgIn2l);

  // compare results
  for(unsigned int y=0; y<rgOutT.size().y(); ++y)
    for(unsigned int x=0; x<rgOutT.size().x(); ++x)
    {
      const Position  pos(x,y);
      const PixelMono p1=rgOutT.getPixel(pos);
      const PixelMono p2=rgOutR.getPixel(pos);
      ensure(p1.get()==p2.get());
    }
}

} // namespace tut

