/*
 * GeneralRaster.hpp
 *
 */
#ifndef INCLUDE_GRAPHICS_FILTER_GENERALRASTER_HPP_FILE
#define INCLUDE_GRAPHICS_FILTER_GENERALRASTER_HPP_FILE

/* public header */

#include "Graphics/Raster.hpp"
#include "Graphics/AutoRaster.hpp"
#include "Graphics/Filter/General.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace Filter
{

//
// generic class - input and output types may differ
//
template <typename TPixOut, typename TPixIn, typename TPixOp>
class GeneralRasterAny: public GeneralAny< Raster<TPixOut>,
                                           Raster<TPixIn>,
                                           TPixOp >
{
public:
  typedef Raster<TPixIn>      TRasterIn;
  typedef Raster<TPixOut>     TRasterOut;
  typedef AutoRaster<TPixIn>  TAutoRasterIn;
  typedef AutoRaster<TPixOut> TAutoRasterOut;

  GeneralRasterAny(const TPixOp &op):
    GeneralAny< Raster<TPixOut>, Raster<TPixIn>, TPixOp >(op)
  {
  }

  TAutoRasterOut filterNew(const TRasterIn &in) const
  {
    TAutoRasterOut ar( in.size() );
    assert( ar.get()!=NULL );
    filter( *ar.get(), in );
    return ar;
  }
}; // class GeneralRasterAny


//
// class version assuming having the same in/out
// raster types.
//
template <typename TPixel, typename TPixOp>
class GeneralRaster: public GeneralRasterAny<TPixel, TPixel, TPixOp>
{
public:
  GeneralRaster(const TPixOp &op):
    GeneralRasterAny<TPixel, TPixel, TPixOp>(op)
  {
  }
}; // class GeneralRaster

} // namespace Filter
} // namespace graphics

#endif

