/*
 * GeneralRaster.t.cpp
 *
 */
#include <tut.h>

#include "Graphics/Filter/GeneralRaster.hpp"


namespace
{
struct SomeTestOperation
{
  Graphics::PixelMono operator()(const Graphics::Position   &pos,
                                 const Graphics::RasterMono &in) const
  {
    const Graphics::PixelMono &pIn=in.getPixel(pos);
    return Graphics::PixelMono( (pIn.get()<255)?(pIn.get()+1):255 );
  }
}; // struct SomeTestOperation

struct SomeTestOperation2
{
  Graphics::PixelRGB operator()(const Graphics::Position   &pos,
                                const Graphics::RasterMono &in) const
  {
    const int             i=in.getPixel(pos).get();
    const Graphics::color c=std::min(255-2, i);
    return Graphics::PixelRGB(c+0, c+1, c+2);
  }
}; // struct SomeTestOperation2

} // unnamed namespace


namespace Graphics
{
namespace Filter
{

struct GeneralRasterTestClass
{
  GeneralRasterTestClass(void):
    _size(4,3),
    _in (_size),
    _out(_size),
    _gf (_sto)
  {
  }

  const Size        _size;
  RasterMono        _in;
  RasterMono        _out;
  SomeTestOperation _sto;
  GeneralRaster<PixelMono, SomeTestOperation> _gf;
};

} // namespace Filter
} // namespace Graphics


namespace tut
{
typedef Graphics::Filter::GeneralRasterTestClass TestClass;
typedef test_group<TestClass> factory;
typedef factory::object testObj;
} // namespace tut


namespace
{
tut::factory tf("Graphics/Filter/GeneralRaster");
}


using namespace Graphics;
using namespace Graphics::Filter;

namespace tut
{

// test if it works in regular conditions
template<>
template<>
void testObj::test<1>(void)
{
  // fransform manualy
  for(unsigned int y=0; y<_in.size().y(); ++y)
    for(unsigned int x=0; x<_in.size().x(); ++x)
    {
      const Position pos(x,y);
      _out.setPixel( pos, _sto(pos, _in) );
    }

  // make automatic transform
  AutoRaster<PixelMono> ar=_gf.filterNew(_in);

  // compare automatic and manual results
  for(unsigned int y=0; y<ar->size().y(); ++y)
    for(unsigned int x=0; x<ar->size().x(); ++x)
    {
      const Position   pos(x,y);
      const PixelMono &p1=ar->getPixel(pos);
      const PixelMono &p2=_out.getPixel(pos);
      ensure(p1.get()==p2.get());
    }
}

// check for throw on size mismatch
template<>
template<>
void testObj::test<2>(void)
{
  RasterMono rg( Size(2,9) );
  try
  {
    _gf.filter(rg, _in);
    fail("comparing output image size with input didn't failed");
  }
  catch(const Graphics::Exception &ex)
  {
    // this is expected behavior
  }
}

// enshure that template arguments sequence is correct
template<>
template<>
void testObj::test<3>(void)
{
  RasterMono rg( Size(1,1) );
  RasterRGB  rc( Size(1,1) );
  const PixelMono pm(120);
  rg.fill(pm);
  GeneralRasterAny<PixelRGB, PixelMono, SomeTestOperation2>
    ga( (SomeTestOperation2()) );

  ga.filter(rc, rg);
  const PixelRGB p=rc.getPixel( Position(0,0) );
  ensure( p.getR()==pm.get()+0 &&
          p.getG()==pm.get()+1 &&
          p.getB()==pm.get()+2    );
}

} // namespace tut

