/*
 * GeneralRasterTransformation.hpp
 *
 * transformation of image filtering
 *
 */
#ifndef INCLUDE_GRAPHICS_FILTER_GENERALRASTERTRANSFORMATION_HPP_FILE
#define INCLUDE_GRAPHICS_FILTER_GENERALRASTERTRANSFORMATION_HPP_FILE

/* public header */

#include "Graphics/SharedRaster.hpp"
#include "Graphics/Filter/GeneralRaster.hpp"
#include "Graphics/Transform/Transformation.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace Filter
{

template<typename TPixel, typename TPixOp>
class GeneralRasterTransformation: public
                    Graphics::Transform::Transformation
{
public:
  typedef SharedRaster<TPixel> TShrRaster;

  GeneralRasterTransformation(TPixOp     pixOp,
                              TShrRaster rOut,
                              TShrRaster rIn):
    _gf  (pixOp),
    _rIn (rIn),
    _rOut(rOut)
  {
  }

  inline TShrRaster getRasterIn(void) const
  {
    return _rIn;
  }
  inline TShrRaster getRasterOut(void) const
  {
    return _rOut;
  }

  virtual void transform(void)
  {
    _gf.filter( *_rOut.get(), *_rIn.get() );
  }

private:
  GeneralRaster<TPixel, TPixOp> _gf;
  TShrRaster                    _rIn;
  TShrRaster                    _rOut;
}; // class GeneralRasterTransformation

} // namespace Filter
} // namespace Graphics

#endif

