/*
 * GeneralRasterTwo.hpp
 *
 */
#ifndef INCLUDE_GRAPHICS_FILTER_GENERALRASTERTWO_HPP_FILE
#define INCLUDE_GRAPHICS_FILTER_GENERALRASTERTWO_HPP_FILE

/* public header */

#include <boost/noncopyable.hpp>
#include <cassert>

#include "Graphics/Filter/GeneralTwo.hpp"
#include "Graphics/Raster.hpp"
#include "Graphics/AutoRaster.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace Filter
{

//
// generic class - input and output types may differ
//
template<typename TPixOut, typename TPixIn1, typename TPixIn2, typename TPixOp>
class GeneralRasterTwoAny: public GeneralTwoAny< Raster<TPixOut>,
                                                 Raster<TPixIn1>,
                                                 Raster<TPixIn2>,
                                                 TPixOp >
{
public:
  typedef Raster<TPixIn1>     TRasterIn1;
  typedef Raster<TPixIn2>     TRasterIn2;
  typedef Raster<TPixOut>     TRasterOut;
  typedef AutoRaster<TPixOut> TAutoRasterOut;

  GeneralRasterTwoAny(const TPixOp &op):
    GeneralTwoAny< Raster<TPixOut>, Raster<TPixIn1>,
                   Raster<TPixIn2>, TPixOp >(op)
  {
  }

  TAutoRasterOut filterNew(const TRasterIn1 &in1,
                           const TRasterIn2 &in2)
  {
    if( in1.size()!=in2.size() )
      throw Graphics::Exception("GeneralRasterTwoAny::filterNew(): "
                                "input images sizes doesn't match");
    TAutoRasterOut ar( in1.size() );
    assert( ar.get()!=NULL );
    filter( *ar.get(), in1, in2 );
    return ar;
  }
}; // class GeneralRasterTwoAny


//
// class version assuming to have the same in1/in2/out
// raster types.
//
template<typename TPixel, typename TPixOp>
class GeneralRasterTwo: public
            GeneralRasterTwoAny<TPixel, TPixel, TPixel, TPixOp>
{
public:
  GeneralRasterTwo(const TPixOp &op):
    GeneralRasterTwoAny<TPixel, TPixel, TPixel, TPixOp>(op)
  {
  }
}; // class General

} // namespace Filter
} // namespace graphics

#endif

