/*
 * GeneralRasterTwoTransformation.hpp
 *
 * transformation of image filtering
 *
 */
#ifndef INCLUDE_GRAPHICS_FILTER_GENERALRASTERTWOTRANSFORMATION_HPP_FILE
#define INCLUDE_GRAPHICS_FILTER_GENERALRASTERTWOTRANSFORMATION_HPP_FILE

/* public header */

#include "Graphics/SharedRaster.hpp"
#include "Graphics/Filter/GeneralRasterTwo.hpp"
#include "Graphics/Transform/Transformation.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace Filter
{

template<typename TPixel, typename TPixOp>
class GeneralRasterTwoTransformation: public
                Graphics::Transform::Transformation
{
public:
  typedef SharedRaster<TPixel> TShrRaster;

  GeneralRasterTwoTransformation(const TPixOp &pixOp,
                                 TShrRaster    rOut,
                                 TShrRaster    rIn1,
                                 TShrRaster    rIn2):
    _gf  (pixOp),
    _rIn1(rIn1),
    _rIn2(rIn2),
    _rOut(rOut)
  {
  }

  inline TShrRaster getRasterIn1(void) const
  {
    return _rIn1;
  }
  inline TShrRaster getRasterIn2(void) const
  {
    return _rIn2;
  }
  inline TShrRaster getRasterOut(void) const
  {
    return _rOut;
  }

  virtual void transform(void)
  {
    _gf.filter( *_rOut.get(), *_rIn1.get(), *_rIn2.get() );
  }

private:
  GeneralRasterTwo<TPixel, TPixOp> _gf;
  TShrRaster                       _rIn1;
  TShrRaster                       _rIn2;
  TShrRaster                       _rOut;
}; // class GeneralRasterTwoTransformation

} // namespace Filter
} // namespace Graphics

#endif

