/*
 * GeneralRasterTwoTransformation.t.cpp
 *
 */
#include <tut.h>
#include <algorithm>

#include "Graphics/Filter/GeneralRasterTwoTransformation.hpp"

namespace Graphics
{
namespace Filter
{

struct GeneralRasterTwoTransformationTestClass
{
};

} // namespace Filter
} // namespace Graphics


namespace tut
{
typedef Graphics::Filter::GeneralRasterTwoTransformationTestClass TestClass;
typedef test_group<TestClass> factory;
typedef factory::object testObj;
} // namespace tut


namespace
{
tut::factory tf("Graphics/Filter/GeneralRasterTwoTransformation");
}


using namespace boost;
using namespace Graphics;
using namespace Graphics::Filter;

namespace
{
struct SomeTestOperation
{
  Graphics::PixelMono operator()(const Graphics::Position   &pos,
                                 const Graphics::RasterMono &in1,
                                 const Graphics::RasterMono &in2) const
  {
    const Graphics::PixelMono &pIn1=in1.getPixel(pos);
    const Graphics::PixelMono &pIn2=in2.getPixel(pos);
    return Graphics::PixelMono( std::min( pIn1.get(), pIn2.get() ) );
  }
}; // struct SomeTestOperation
} // unnamed namespace



namespace tut
{

// test filtering via Transformation and check if it matches
template<>
template<>
void testObj::test<1>(void)
{
  // some random input raster
  const Size size(10,20);
  SharedRasterMono rgIn1 (size);
  SharedRasterMono rgIn2 (size);
  SharedRasterMono rgOutT(size);
  // output raster after 'regular' filtering
  SharedRasterMono rgOutR(size);

  // use transformation
  SomeTestOperation sto;
  GeneralRasterTwoTransformation<PixelMono, SomeTestOperation>
    gft(sto, rgOutT, rgIn1, rgIn2);
  gft.transform();

  // transform manualy
  GeneralRasterTwo<PixelMono, SomeTestOperation> grt(sto);
  grt.filter( *rgOutR.get(), *rgIn1.get(), *rgIn2.get()  );

  // compare results
  for(unsigned int y=0; y<rgOutT->size().y(); ++y)
    for(unsigned int x=0; x<rgOutT->size().x(); ++x)
    {
      const Position  pos(x,y);
      const PixelMono p1=rgOutT->getPixel(pos);
      const PixelMono p2=rgOutR->getPixel(pos);
      ensure(p1.get()==p2.get());
    }
}

} // namespace tut

