/*
 * GeneralTwo.hpp
 *
 * this is generic General class.
 *
 * generic filter operation - iterates throught
 * all pixels of two input rasters and performs
 * user-defined operation on both of them.
 *
 * NOTE: order in which pixels will be processed
 *       is NOT defined, and therefore it should
 *       not be assumed anyhow (it might change
 *       in a future).
 *
 */
#ifndef INCLUDE_GRAPHICS_FILTER_GENERALTWO_HPP_FILE
#define INCLUDE_GRAPHICS_FILTER_GENERALTWO_HPP_FILE

/* public header */

#include <boost/noncopyable.hpp>

#include "Graphics/Position.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace Filter
{

//
// the TPixOp function object should have defined
// following operator:
//
//      TPixel operator()(const Position &pos, const TRaster &in)
//


//
// generic class - input and output types may differ
//
template<typename TImageOut,
         typename TImageIn1,
         typename TImageIn2,
         typename TPixOp>
class GeneralTwoAny: private boost::noncopyable
{
public:
  GeneralTwoAny(const TPixOp &op):
    _op(op)
  {
  }

  void filter(TImageOut &out, const TImageIn1 &in1,
                              const TImageIn2 &in2) const
  {
    if( in1.size()!=out.size() || in1.size()!=in2.size() )
      throw Graphics::Exception("GeneralTwo::filter(): "
                                "input and output images sizes doesn't match");
    // iterate throught image
    const size_t sizeX=in1.size().x();
    const size_t sizeY=in1.size().y();
    for(size_t y=0; y<sizeY; ++y)
      for(size_t x=0; x<sizeX; ++x)
      {
        const Position pos(x,y);
        out.setPixel( pos, _op(pos, in1,in2) );
      }
  }

  inline const TPixOp &getPixOp(void) const
  {
    return _op;
  }

private:
  const TPixOp _op;
}; // class GeneralAny


//
// class version assuming to have the same in1/in2/out
// raster types.
//
template<typename TImage, typename TPixOp>
class GeneralTwo: public GeneralTwoAny<TImage, TImage, TImage, TPixOp>
{
public:
  GeneralTwo(const TPixOp &op):
    GeneralTwoAny<TImage, TImage, TImage, TPixOp>(op)
  {
  }
}; // class General

} // namespace Filter
} // namespace graphics

#endif

