/*
 * GeneralTwo.t.cpp
 *
 */
#include <tut.h>

#include <algorithm>

#include "Graphics/Raster.hpp"
#include "Graphics/AutoRaster.hpp"
#include "Graphics/Filter/GeneralTwo.hpp"


namespace
{
struct SomeTestOperation
{
  Graphics::PixelMono operator()(const Graphics::Position   &pos,
                                 const Graphics::RasterMono &in1,
                                 const Graphics::RasterMono &in2) const
  {
    const Graphics::PixelMono &pIn1=in1.getPixel(pos);
    const Graphics::PixelMono &pIn2=in2.getPixel(pos);
    return Graphics::PixelMono( std::min(pIn1.get(), pIn2.get()) );
  }


  Graphics::PixelMono operator()(const Graphics::Position   &pos,
                                 const Graphics::RasterMono &in1,
                                 const Graphics::RasterRGB  &in2) const
  {
    const Graphics::PixelMono &pIn1=in1.getPixel(pos);
    const Graphics::PixelRGB  &pIn2=in2.getPixel(pos);
    Graphics::color            cg  =(pIn2.getR()+pIn2.getG()+pIn2.getB())/3;
    return Graphics::PixelMono( std::min(pIn1.get(), cg) );
  }
}; // struct SomeTestOperation

} // unnamed namespace


namespace Graphics
{
namespace Filter
{

struct GeneralTwoTestClass
{
  GeneralTwoTestClass(void):
    _size(4,3),
    _in1(_size),
    _in2(_size),
    _out(_size),
    _gf (_sto)
  {
  }

  const Size        _size;
  RasterMono        _in1;
  RasterMono        _in2;
  RasterMono        _out;
  SomeTestOperation _sto;
  GeneralTwo<Raster<PixelMono>, SomeTestOperation> _gf;
};

} // namespace Filter
} // namespace Graphics


namespace tut
{
typedef Graphics::Filter::GeneralTwoTestClass TestClass;
typedef test_group<TestClass> factory;
typedef factory::object testObj;
} // namespace tut


namespace
{
tut::factory tf("Graphics/Filter/GeneralTwo");
}


using namespace Graphics;
using namespace Graphics::Filter;

namespace tut
{

// test if it works in regular conditions
template<>
template<>
void testObj::test<1>(void)
{
  // fransform manualy
  for(unsigned int y=0; y<_in1.size().y(); ++y)
    for(unsigned int x=0; x<_in1.size().x(); ++x)
    {
      const Position pos(x,y);
      _out.setPixel( pos, _sto(pos, _in1, _in2) );
    }

  // make automatic transform
  RasterMono out2( _in1.size() );
  _gf.filter(out2, _in1, _in2);

  // compare automatic and manual results
  for(unsigned int y=0; y<out2.size().y(); ++y)
    for(unsigned int x=0; x<out2.size().x(); ++x)
    {
      const Position   pos(x,y);
      const PixelMono &p1=out2.getPixel(pos);
      const PixelMono &p2=_out.getPixel(pos);
      ensure(p1.get()==p2.get());
    }
}

// check for throw on size mismatch
template<>
template<>
void testObj::test<2>(void)
{
  RasterMono rg( Size(2,9) );

  try
  {
    _gf.filter(rg, _in1, _in2);
    fail("comparing output image size with input didn't failed");
  }
  catch(const Graphics::Exception &ex)
  {
    // this is expected behavior
  }

  try
  {
    _gf.filter(_in1, _in1, rg);
    fail("comparing output image size with input didn't failed");
  }
  catch(const Graphics::Exception &ex)
  {
    // this is expected behavior
  }
}

// enshure that template arguments sequence is correct.
// it is just fine enought that this compiles.
template<>
template<>
void testObj::test<3>(void)
{
  RasterMono rOut( Size(1,1) );
  RasterMono rg  ( Size(1,1) );
  RasterRGB  rc  ( Size(1,1) );

  GeneralTwoAny< Raster<PixelMono>, Raster<PixelMono>,
                 Raster<PixelRGB>,  SomeTestOperation >
    ga( (SomeTestOperation()) );
  ga.filter(rOut, rg,rc);
}

} // namespace tut

