/*
 * RasterAdder.hpp
 *
 * raster adding implementation.
 *
 */
#ifndef INCLUDE_GRAPHICS_FILTER_RASTERADDER_HPP_FILE
#define INCLUDE_GRAPHICS_FILTER_RASTERADDER_HPP_FILE

/* public header */

#include <algorithm>

#include "Graphics/PixelTransform/PixelAdder.hpp"
#include "Graphics/Filter/GeneralRasterTwo.hpp"
#include "Graphics/Filter/GeneralRasterTwoTransformation.hpp"


namespace Graphics
{
namespace Filter
{

// main processing unit - computes Median for single pixel
struct RasterAdderProcessor
{
  template<typename TPixel>
  inline TPixel operator()(const Position &pos,
                           const Raster<TPixel> &in1,
                           const Raster<TPixel> &in2) const
  {
    const TPixel p1=in1.getPixel(pos);
    const TPixel p2=in2.getPixel(pos);
    return _pa(p1,p2);
  }

  const Graphics::PixelTransform::PixelAdder _pa;
}; // struct RasterAdderProcessor


//
// processing unit to use for raster adding
//
template<typename TPixel>
class RasterAdder: public GeneralRasterTwo<TPixel, RasterAdderProcessor>
{
public:
  RasterAdder(void):
    GeneralRasterTwo<PixelMono, RasterAdderProcessor>
        ( RasterAdderProcessor() )
  {
  }
}; // class RasterAdder


//
// raster adding transformation
//
template<typename TPixel>
class RasterAdderTransformation: public
        GeneralRasterTwoTransformation<TPixel, RasterAdderProcessor>
{
private:
  typedef GeneralRasterTwoTransformation<TPixel,
                            RasterAdderProcessor> Base;
  typedef typename Base::TShrRaster               TShrRaster;

public:
  RasterAdderTransformation(      TShrRaster out,
                            const TShrRaster in1,
                            const TShrRaster in2):
    Base( RasterAdderProcessor(), out, in1, in2 )
  {
  }
}; // class RasterAdderTransformation

} // namespace Filter
} // namespace Graphics

#endif

