/*
 * RasterDiffer.hpp
 *
 * raster differ implementaion
 *
 */
#ifndef INCLUDE_GRAPHICS_FILTER_RASTERDIFFER_HPP_FILE
#define INCLUDE_GRAPHICS_FILTER_RASTERDIFFER_HPP_FILE

/* public header */

#include <algorithm>

#include "Graphics/PixelTransform/PixelDiffer.hpp"
#include "Graphics/Filter/GeneralRasterTwo.hpp"
#include "Graphics/Filter/GeneralRasterTwoTransformation.hpp"


namespace Graphics
{
namespace Filter
{

// main processing unit - computes Median for single pixel
struct RasterDifferProcessor
{
  template<typename TPixel>
  inline TPixel operator()(const Position &pos,
                           const Raster<TPixel> &in1,
                           const Raster<TPixel> &in2) const
  {
    const TPixel p1=in1.getPixel(pos);
    const TPixel p2=in2.getPixel(pos);
    return _pd(p1,p2);
  }

  const Graphics::PixelTransform::PixelDiffer _pd;
}; // struct RasterDifferProcessor


//
// processing unit to use for raster adding
//
template<typename TPixel>
class RasterDiffer: public
               GeneralRasterTwo<TPixel, RasterDifferProcessor>
{
public:
  RasterDiffer(void):
    GeneralRasterTwo<PixelMono, RasterDifferProcessor>
            ( RasterDifferProcessor() )
  {
  }
}; // class RasterDiffer


//
// raster differ transformation
//
template<typename TPixel>
class RasterDifferTransformation: public
        GeneralRasterTwoTransformation<TPixel, RasterDifferProcessor>
{
private:
  typedef GeneralRasterTwoTransformation<TPixel,
                            RasterDifferProcessor> Base;
  typedef typename Base::TShrRaster                TShrRaster;

public:
  RasterDifferTransformation(      TShrRaster out,
                             const TShrRaster in1,
                             const TShrRaster in2):
    Base( RasterDifferProcessor(), out, in1, in2 )
  {
  }
}; // class RasterDifferTransformation

} // namespace Filter
} // namespace Graphics

#endif

