/*
 * RasterDiffer.t.cpp
 *
 */
#include <tut.h>

#include "Graphics/Filter/RasterDiffer.hpp"

namespace Graphics
{
namespace Filter
{

struct RasterDifferTestClass
{
  RasterDifferTestClass(void):
    _size(1,2),
    _r1 (_size),
    _r2 (_size),
    _out(_size)
  {
  }

  const Size _size;
  RasterMono _r1;
  RasterMono _r2;
  RasterMono _out;
};

} // namespace Filter
} // namespace Graphics


namespace tut
{
typedef Graphics::Filter::RasterDifferTestClass TestClass;
typedef test_group<TestClass> factory;
typedef factory::object testObj;
} // namespace tut


namespace
{
tut::factory tf("Graphics/Filter/RasterDiffer");
}


using namespace Graphics;
using namespace Graphics::Filter;

namespace tut
{

// test if it works in regular conditions
template<>
template<>
void testObj::test<1>(void)
{
  {
    const Position pos(0,0);
    _r1.setPixel( pos, PixelMono( 30) );
    _r2.setPixel( pos, PixelMono(100) );
  }
  {
    const Position pos(0,1);
    _r1.setPixel( pos, PixelMono(30) );
    _r2.setPixel( pos, PixelMono(50) );
  }

  RasterDiffer<PixelMono>().filter(_out, _r1,_r2);  // add two images
  // check if results matches
  {
    const Position pos(0,0);
    ensure( _out.getPixel(pos).get()==70 );
  }
  {
    const Position pos(0,1);
    ensure( _out.getPixel(pos).get()==20 );
  }
}

// check for throw on size mismatch
template<>
template<>
void testObj::test<2>(void)
{
  RasterMono rg( Size(2,9) );
  try
  {
    RasterDiffer<PixelMono>().filter(rg, _r1,_r2);
    fail("comparing output image size with input sizes didn't failed");
  }
  catch(const Graphics::Exception &ex)
  {
    // this is expected behavior
  }
  try
  {
    RasterDiffer<PixelMono>().filter(_out, rg,_r2);
    fail("comparing input images sizes didn't failed");
  }
  catch(const Graphics::Exception &ex)
  {
    // this is expected behavior
  }
}

// try building transformation
template<>
template<>
void testObj::test<3>(void)
{
  SharedRasterMono rOut(_size);
  SharedRasterMono rIn1(_size);
  SharedRasterMono rIn2(_size);
  RasterDifferTransformation<PixelMono> rat(rOut, rIn1, rIn2);
}

} // namespace tut

