/*
 * SobelEdge.hpp
 *
 * Sobel edge detector implementation
 *
 */
#ifndef INCLUDE_GRAPHICS_FILTER_SOBELEDGE_HPP_FILE
#define INCLUDE_GRAPHICS_FILTER_SOBELEDGE_HPP_FILE

/* public header */

#include "Graphics/Filter/GeneralLayer.hpp"
#include "Graphics/Filter/GeneralLayerTransformation.hpp"
#include "Graphics/Filter/SobelEdgeProcessor.hpp"


namespace Graphics
{
namespace Filter
{

//
// processing unit to use for median filtering
//
template <typename TPixel>
class SobelEdge: public GeneralLayer<TPixel, SobelEdgeProcessor<TPixel> >
{
public:
  SobelEdge(void):
    GeneralLayer<TPixel, SobelEdgeProcessor<TPixel> >
        ( (SobelEdgeProcessor<TPixel>()) )
  {
  }
}; // class SobelEdge


//
// sobel edge detector transformation
//
template <typename TPixel>
class SobelEdgeTransformation: public
        GeneralLayerTransformation<TPixel, SobelEdgeProcessor<TPixel> >
{
private:
  typedef SobelEdgeProcessor<TPixel>                TProc;
  typedef GeneralLayerTransformation<TPixel, TProc> TBase;
  typedef RasterLayer<TPixel>                       TRasterLayer;

public:
  SobelEdgeTransformation(      TRasterLayer out,
                          const TRasterLayer in):
    TBase( (TProc()), out, in )
  {
  }
}; // class SobelEdgeTransformation

} // namespace Filter
} // namespace Graphics

#endif

