/*
 * SobelEdge.mt.cpp
 *
 */

#include <iostream>
#include <cassert>

#include "Graphics/Filter/SobelEdge.hpp"
#include "Graphics/Storage/Storers.hpp"

const char inPath[]  ="testdata/edges.jpg";
const char outPathC[]="testdata/sobel_edge_out_C.jpg";
const char outPathM[]="testdata/sobel_edge_out_M.jpg";

using namespace std;
using namespace boost;
using namespace Graphics;
using namespace Graphics::Filter;
using namespace Graphics::Storage;


int main(int argc, char **argv)
{
  if(argc!=1)
    cerr<<argv[0]<<": no parameters are required - skipping imput\n";

  try
  {
    // read image from file
    shared_ptr<RasterRGB> rIn  ( new RasterRGB( Size(1,1) ) );
    StorerJPG             sjIn ( string(inPath), rIn );
    sjIn.read();

    // prepare output image
    shared_ptr<RasterRGB> rOutC( new RasterRGB( rIn->size() ) );

    // get edges from color input image
    SobelEdge<PixelRGB>   seC;
    const PixelRGB::Layer l[]={
                                PixelRGB::Layer::R,
                                PixelRGB::Layer::G,
                                PixelRGB::Layer::B
                              };
    // loop through all layers
    for(size_t i=0; i<sizeof(l)/sizeof(l[0]); ++i)
    {
      RasterLayerRGB in =makeRasterLayerRGB(rIn.get(),   l[i]);
      RasterLayerRGB out=makeRasterLayerRGB(rOutC.get(), l[i]);
      // filterint itself
      seC.filter(out, in);
    }

    // write result to file
    StorerJPG             sjOutC( string(outPathC), rOutC );
    assert( rIn->size()==rOutC->size() );
    sjOutC.write();

    SobelEdge<PixelMono>  seM;
  }
  catch(const std::exception &ex)
  {
    cerr<<argv[0]<<": std::exception: "<<ex.what()<<endl;
    return 1;
  }
  catch(...)
  {
    cerr<<argv[0]<<": unknown exception caught..."<<endl;
    return 2;
  }

  return 0;
}

