/*
 * Thresholder.hpp
 *
 * class performing thresholding of monochromatic images
 *
 */
#ifndef INCLUDE_GRAPHICS_FILTER_THRESHOLDER_HPP_FILE
#define INCLUDE_GRAPHICS_FILTER_THRESHOLDER_HPP_FILE

/* public header */

#include "Graphics/Filter/GeneralLayer.hpp"
#include "Graphics/Filter/GeneralLayerTransformation.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace Filter
{

template<typename TPixel>
class ThresholderProcessor
{
public:
  typedef typename TPixel::ColorType   ColorType;
  typedef          RasterLayer<TPixel> TRasterLayer;

  ThresholderProcessor(ColorType threshold):
    _threshold(threshold)
  {
  }

  // image is now binary!
  inline bool operator()(const Position     &pos,
                         const TRasterLayer &rl) const
  {
    return rl.getPixel(pos) > _threshold;
  }

private:
  ColorType _threshold;
}; // class ThresholderProcessor



//
// binarises the image based on given threshold value
//
template<typename TPixel>
class Thresholder: public GeneralLayerAny<PixelBinary, TPixel,
                            ThresholderProcessor<TPixel> >
{
public:
  typedef typename TPixel::ColorType   ColorTypeIn;
  typedef RasterLayer<TPixel>          TRasterLayerIn;
  typedef RasterLayer<PixelBinary>     TRasterLayerOut;
  typedef ThresholderProcessor<TPixel> TProc;

  Thresholder(ColorTypeIn threshold):
    GeneralLayerAny<PixelBinary, TPixel, TProc>
            ( (TProc(threshold)) )
  {
  }
}; // class Thresholder


//
// transformation for thresholder
//

template<typename TPixel>
class ThresholderTransformation: public
            GeneralLayerAnyTransformation<PixelBinary, TPixel,
                                          ThresholderProcessor<TPixel> >
{
public:
  typedef typename TPixel::ColorType   ColorTypeIn;
  typedef RasterLayer<TPixel>          TRasterLayerIn;
  typedef RasterLayer<PixelBinary>     TRasterLayerOut;
  typedef ThresholderProcessor<TPixel> TProc;

  ThresholderTransformation(ColorTypeIn     threshold,
                            TRasterLayerOut rlOut,
                            TRasterLayerIn  rlIn):
    GeneralLayerAnyTransformation<PixelBinary, TPixel, TProc>
      ( (TProc(threshold)), rlOut, rlIn )
  {
  }

}; // class ThresholderTransformation

} // namespace Filter
} // namespace Graphics

#endif

