/*
 * Thresholder.t.cpp
 *
 */
#include <tut.h>

#include "Graphics/Filter/Thresholder.hpp"

namespace Graphics
{
namespace Filter
{

struct ThresholderTestClass
{
  ThresholderTestClass():
    _size ( Size(2,3) ),
    _rIn  (_size),
    _rInl ( makeRasterLayerMono(&_rIn) ),
    _rOut (_size),
    _rOutl( makeRasterLayerBinary(&_rOut) )
  {
    _rIn.setPixel( Position(0,0), PixelMono( 10) );
    _rIn.setPixel( Position(0,1), PixelMono(255) );
    _rIn.setPixel( Position(0,2), PixelMono(210) );
    _rIn.setPixel( Position(1,0), PixelMono( 90) );
    _rIn.setPixel( Position(1,1), PixelMono(110) );
    _rIn.setPixel( Position(1,2), PixelMono( 50) );
  }

  const Size        _size;
  RasterMono        _rIn;
  RasterLayerMono   _rInl;
  RasterBinary      _rOut;
  RasterLayerBinary _rOutl;
};

} // namespace Transform
} // namespace Graphics


namespace tut
{
typedef Graphics::Filter::ThresholderTestClass TestClass;
typedef test_group<TestClass> factory;
typedef factory::object testObj;
} // namespace tut


namespace
{
tut::factory tf("Graphics/Filter/Thresholder");
}


using namespace Graphics;
using namespace Graphics::Filter;

namespace tut
{

// test if transformation works
template<>
template<>
void testObj::test<1>(void)
{
  Thresholder<PixelMono> th(127);
  th.filter(_rOutl, _rInl);

  // check
  for(unsigned int y=0; y<_rIn.size().y(); ++y)
    for(unsigned int x=0; x<_rIn.size().x(); ++x)
    {
      const Position        pos(x,y);
      const Graphics::color cIn =_rInl.getPixel(pos);
      const bool            cOut=_rOutl.getPixel(pos);
      if(cIn==127)  // threshold level can give any value
        continue;
      if(cIn>127)
        ensure(cOut==true);
      if(cIn<127)
        ensure(cOut==false);
    }
}

// test RGB image thresholding
template<>
template<>
void testObj::test<2>(void)
{
  RasterRGB      r (_size);
  RasterLayerRGB rl(&r, PixelRGB::Layer::R);
  Thresholder<PixelRGB> th(22);
  th.filter(_rOutl, rl);
}

// test if transformation gives good results
template<>
template<>
void testObj::test<3>(void)
{
  // test thresholding
  Thresholder<PixelMono> th(127);
  th.filter(_rOutl, _rInl);

  // test transformation
  RasterBinary      rOut2 ( _rOut.size() );
  RasterLayerBinary rOut2l( makeRasterLayerBinary(&rOut2) );
  ThresholderTransformation<PixelMono> tht(127, rOut2l, _rInl);
  tht.transform();

  // check if both are equal
  for(unsigned int y=0; y<_rIn.size().y(); ++y)
    for(unsigned int x=0; x<_rIn.size().x(); ++x)
    {
      const Position pos(x,y);
      const bool     cOut1=rOut2l.getPixel(pos);
      const bool     cOut2=_rOutl.getPixel(pos);
      ensure(cOut1==cOut2);
    }
}

} // namespace tut

