/*
 * PixelConverter.hpp
 *
 */
#ifndef INCLUDE_GRAPHICS_HELPERS_MAGICK_PIXELCONVERTER_HPP_FILE
#define INCLUDE_GRAPHICS_HELPERS_MAGICK_PIXELCONVERTER_HPP_FILE

#include <Magick++.h>

#include "Graphics/Pixel.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace Helpers
{
namespace MagickPP
{

// template for convertion between build-in pixel types
struct PixelConverter
{
  template<typename TPixOut, typename TPixIn>
  inline TPixOut operator()(const TPixIn &in) const
  {
    TPixOut out;        // create
    (*this)(out, in);   // convert
    return out;         // and return
  }

  // Gray -> RGB
  inline void operator()(PixelRGB &out, const Magick::ColorRGB &in) const
  {
    out.setR( static_cast<unsigned char>(in.red()  *255) );
    out.setG( static_cast<unsigned char>(in.green()*255) );
    out.setB( static_cast<unsigned char>(in.blue() *255) );
  }

  // RGB -> Gray
  inline void operator()(Magick::ColorRGB &out, const PixelRGB &in) const
  {
    out=Magick::ColorRGB( in.getR()/255.0,
                          in.getG()/255.0,
                          in.getB()/255.0 );
    // why above code works, and below doesn't?!
    // bug to Magic++ has been raported
    /*
    out.red  (in._r/255.0);
    out.green(in._g/255.0);
    out.blue (in._b/255.0);
    */
  }
}; // struct PixelConverter

} // namespace MagickPP
} // namespace Helpers
} // namespace Graphics

#endif

