/*
 * RasterConverter.hpp
 *
 */
#ifndef INCLUDE_GRAPHICS_HELPERS_MAGICKPP_RASTERCONVERTER_HPP_FILE
#define INCLUDE_GRAPHICS_HELPERS_MAGICKPP_RASTERCONVERTER_HPP_FILE

#include <Magick++.h>

#include "Graphics/Raster.hpp"
#include "Graphics/Helpers/MagickPP/PixelConverter.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace Helpers
{
namespace MagickPP
{

// template for convertion between different raster types
// (based on known, build-in pixel types).
class RasterConverter
{
public:
  template<typename TRasterOut, typename TRasterIn>
  inline void operator()(TRasterOut &out, const TRasterIn &in) const
  {
    if( getSize(out)!=getSize(in) )
      throw Exception("Helpers::MagickPP::RasterConverter::operator(): "
                      "input and output raster sizes doesn't match");

    // begin convertion pixel by pixel
    const unsigned int size_y=getSize(in).y();
    const unsigned int size_x=getSize(in).x();
    // loop throught pixels
    for(unsigned int y=0; y<size_y; ++y)
      for(unsigned int x=0; x<size_x; ++x)
      {
        const Position  pos(x,y);
        setPixel(out, getPixel(in, pos), pos);
      }
  }

private:
  PixelConverter _pc;

  // pixel getters
  inline Magick::ColorRGB getPixel(const Magick::Image &img,
                                   const Position      &pos) const
  {
    return img.pixelColor( pos.x(), pos.y() );
  }
  inline PixelRGB getPixel(const RasterRGB &img,
                           const Position  &pos) const
  {
    return img.getPixel(pos);
  }

  // pixel setters
  inline void setPixel(      Magick::Image &img,
                       const PixelRGB      &pix,
                       const Position      &pos) const
  {
    Magick::ColorRGB pixOut=_pc.operator()<Magick::ColorRGB,PixelRGB>(pix);
    img.pixelColor( pos.x(), pos.y(), pixOut );
  }
  inline void setPixel(      RasterRGB        &img,
                       const Magick::ColorRGB &pix,
                       const Position         &pos) const
  {
    PixelRGB pixOut=_pc.operator()<PixelRGB,Magick::ColorRGB>(pix);
    img.setPixel(pos, pixOut);
  }

  // size getters
  inline Size getSize(const Magick::Image &img) const
  {
    return Size( img.columns(), img.rows() );
  }
  inline Size getSize(const RasterRGB &img) const
  {
    return img.size();
  }
}; // class RasterConverter

} // namespace MagickPP
} // namespace Helpers
} // namespace Graphics

#endif

