/*
 * RasterConverter.t.cpp
 *
 */
#include <tut.h>
#include <math.h>

#include "Graphics/Helpers/MagickPP/RasterConverter.hpp"

namespace Graphics
{
namespace Helpers
{
namespace MagickPP
{

struct RasterConverterTestClass
{
  RasterConverterTestClass():
    _s(2,3),
    _rst(_s),
    _img( Magick::Geometry(_s.x(),_s.y()), Magick::ColorRGB(0,0,0) ),
    _eps(0.01)
  {
    // at least one must differ
    _img.pixelColor(1,2, Magick::ColorRGB(0.0, 1.0, 0.0) );
    _rst.setPixel( Position(1,2), PixelRGB(255,0,255) );
  }

  bool compare(void) const
  {
    for(unsigned int y=0; y<_s.y(); ++y)
      for(unsigned int x=0; x<_s.x(); ++x)
      {
        Position       pos(x,y);
        const Magick::ColorRGB p1=_pc.operator()<Magick::ColorRGB,PixelRGB>
                                           (_rst.getPixel(pos));
        const Magick::ColorRGB p2=_img.pixelColor(x,y);

        if( fabs( p1.red()  -p2.red()   )>_eps ||
            fabs( p1.green()-p2.green() )>_eps ||
            fabs( p1.blue() -p2.blue()  )>_eps    )
          return true;
      }
    return true;
  }

  Size           _s;
  RasterRGB      _rst;
  Magick::Image  _img;
  PixelConverter _pc;
  const double   _eps;
};

} // namespace MagickPP
} // namespace Helpers
} // namespace Graphics


namespace tut
{
typedef Graphics::Helpers::MagickPP::RasterConverterTestClass TestClass;
typedef test_group<TestClass> factory;
typedef factory::object testObj;
} // namespace tut


namespace
{
tut::factory tf("Graphics/Helpers/MagickPP/RasterConverter");
}


using namespace Magick;
using namespace Graphics;
using namespace Graphics::Helpers::MagickPP;

namespace tut
{

// converting RasterRGB -> Image
template<>
template<>
void testObj::test<1>(void)
{
  RasterConverter rc;
  rc(_img, _rst);       // convert
  ensure( compare() );  // check if convertion was succesfull
}

// converting Image -> RasterRGB
template<>
template<>
void testObj::test<2>(void)
{
  RasterConverter rc;
  rc(_rst, _img);       // convert
  ensure( compare() );  // check if convertion was succesfull
}

} // namespace tut

