/*
 * Color.hpp
 *
 */
#ifndef INCLUDE_GRAPHICS_HISTOGRAM_COLOR_HPP_FILE
#define INCLUDE_GRAPHICS_HISTOGRAM_COLOR_HPP_FILE

/* public header */

#include <cassert>
#include <string.h>
#include <boost/static_assert.hpp>
#include <boost/type_traits.hpp>

#include "Graphics/Histogram/General.hpp"
#include "Graphics/Histogram/GeneralTransformation.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace Histogram
{

// histogram base on array of Graphics::color values.
template <typename TPixel>
class ColorBase: public General<TPixel, 256>
{
  typedef typename TPixel::ColorType ColorType;
  // check if ColorType is unsigned int (Graphics::color).
  BOOST_STATIC_ASSERT(boost::is_unsigned<ColorType>::value &&
                      sizeof(ColorType)==1);
public:
  explicit ColorBase(const RasterLayer<TPixel> &rl):
    General<TPixel, 256>(rl)
  {
  }
}; // class ColorBase


//
// some common interpretations of Histogram
//
typedef ColorBase<PixelMono> Mono;
typedef ColorBase<PixelRGB>  RGB;


// histogram transformation for color
template <typename TPixel>
class ColorBaseTransformation: public GeneralTransformation<TPixel, 255>
{
public:
  typedef RasterLayer<TPixel> TRasterLayer;

  ColorBaseTransformation(const TRasterLayer &rl):
    GeneralTransformation<TPixel, 255>(rl)
  {
  }
}; // class ColorBaseTransformation


//
// some common interpretations of Histogram
//
typedef ColorBaseTransformation<PixelMono> MonoTransformation;
typedef ColorBaseTransformation<PixelRGB>  RGBTransformation;

} // namespace Histogram
} // namespace Graphics

#endif

