/*
 * General.t.cpp
 *
 */
#include <tut.h>

#include "Graphics/Raster.hpp"
#include "Graphics/Histogram/General.hpp"

namespace Graphics
{
namespace Histogram
{

struct GeneralTestClass
{
  GeneralTestClass():
    _s(2,3),
    _r(_s)
  {
    tut::ensure( sizeof(_colorsR)==_s.x()*_s.y() );
    tut::ensure( sizeof(_colorsG)==_s.x()*_s.y() );
    tut::ensure( sizeof(_colorsB)==_s.x()*_s.y() );
    unsigned char c=0;
    for(unsigned char i=0; i<sizeof(_colorsR); ++i)
    {
      _colorsR[i]=c++;
      _colorsG[i]=c++;
      _colorsB[i]=c++;
    }

    for(unsigned int y=0; y<_s.y(); ++y)
      for(unsigned int x=0; x<_s.x(); ++x)
      {
        unsigned char  pos=y*_s.x()+x;
        const PixelRGB p( _colorsR[pos],
                          _colorsG[pos],
                          _colorsB[pos] );
        _r.setPixel( Position(x,y), p );
      }
  }

  Size          _s;
  RasterRGB     _r;
  unsigned char _colorsR[2*3];
  unsigned char _colorsG[2*3];
  unsigned char _colorsB[2*3];
};

} // namespace Histogram
} // namespace Graphics


namespace tut
{
typedef Graphics::Histogram::GeneralTestClass TestClass;
typedef test_group<TestClass> factory;
typedef factory::object testObj;
} // namespace tut


namespace
{
tut::factory tf("Graphics/Histogram/General");
}


using namespace Graphics;
using namespace Graphics::Histogram;

typedef General<PixelRGB, 256> HistRGB;

namespace tut
{

// test building of histogram
template<>
template<>
void testObj::test<1>(void)
{
  // create layers
  RasterLayerRGB rlR(&_r, PixelRGB::Layer::R);
  RasterLayerRGB rlG(&_r, PixelRGB::Layer::G);
  RasterLayerRGB rlB(&_r, PixelRGB::Layer::B);

  // compute layers' histograms
  HistRGB hR(rlR);
  HistRGB hG(rlG);
  HistRGB hB(rlB);

  // does 'positive' values match?
  unsigned char r[256];
  unsigned char g[256];
  unsigned char b[256];
  memset(r, 0, 256);
  memset(g, 0, 256);
  memset(b, 0, 256);
  for(unsigned int i=0; i<sizeof(_colorsR); ++i)
  {
    // mark non-zeros that are ok
    r[_colorsR[i]]=1;
    g[_colorsG[i]]=1;
    b[_colorsB[i]]=1;
    // check values
    ensure( hR[_colorsR[i]]==1 );
    ensure( hG[_colorsG[i]]==1 );
    ensure( hB[_colorsB[i]]==1 );
  }

  // are others set to 0?
  for(unsigned int i=0; i<256; ++i)
  {
    if(r[i]==0)
      ensure( hR[i]==0 );
    if(g[i]==0)
      ensure( hG[i]==0 );
    if(b[i]==0)
      ensure( hB[i]==0 );
  }
}

// test integrating of histogram
template<>
template<>
void testObj::test<2>(void)
{
  RasterLayerRGB rl(&_r, PixelRGB::Layer::G);

  HistRGB h   (rl);
  HistRGB hInt(rl);

  hInt.integrate();

  // integrate colors to check
  unsigned int intG=0;
  for(unsigned int i=0; i<256; ++i)
  {
    intG+=h[i];
    ensure( intG==hInt[i] );
  }
}

// histogram smoothing
template<>
template<>
void testObj::test<3>(void)
{
  RasterLayerRGB rl(&_r, PixelRGB::Layer::G);
  HistRGB        h1(rl);
  HistRGB        h2(h1);

  h2.smooth();

  // check average difference in color values
  unsigned int diff1=0;
  unsigned int diff2=0;
  for(size_t i=1; i<256; ++i)
  {
    const long d1=h1[i-1]-h1[i];
    const long d2=h2[i-1]-h2[i];
    diff1+=std::abs(d1);
    diff2+=std::abs(d2);
  }
  // smoothed hostogram should have smaller average
  // difference than non-smoothed one.
  ensure("looks like there is something wrong with smooting",
         diff1>=diff2);
}

} // namespace tut

