/*
 * GeneralTransformation.hpp
 *
 * transformation of image to its histogram
 *
 */
#ifndef INCLUDE_GRAPHICS_HISTOGRAM_GENERALTRANSFORMATION_HPP_FILE
#define INCLUDE_GRAPHICS_HISTOGRAM_GENERALTRANSFORMATION_HPP_FILE

/* public header */

#include <boost/scoped_ptr.hpp>

#include "Graphics/Histogram/General.hpp"
#include "Graphics/Transform/Transformation.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace Histogram
{

template <typename TPixel, size_t valuesMax>
class GeneralTransformation: public Graphics::Transform::Transformation
{
public:
  typedef RasterLayer<TPixel>        TRasterLayer;
  typedef General<TPixel, valuesMax> TGeneral;

  explicit GeneralTransformation(const TRasterLayer &rl):
    _rl  (rl),
    _hist(0)
  {
  }

  const TGeneral &getHistogram(void) const
  {
    if( _hist.get()==NULL )
      throw Exception("GeneralTransformation::getHistogram(): "
                      "histogram computation wasn't done yet");
    return *_hist.get();
  }

  virtual void transform(void)
  {
    _hist.reset( new TGeneral(_rl) );
  }

private:
  TRasterLayer                _rl;
  boost::scoped_ptr<TGeneral> _hist;
}; // class GeneralTransformation

} // namespace Histogram
} // namespace Graphics

#endif

