/*
 * GeneralTransformation.t.cpp
 *
 */
#include <tut.h>

#include "Graphics/Histogram/GeneralTransformation.hpp"

namespace Graphics
{
namespace Histogram
{

struct GeneralTransformationTestClass
{
  GeneralTransformationTestClass():
    _s (2,3),
    _r (_s),
    _rl(&_r, PixelRGB::Layer::G)
  {
    tut::ensure( sizeof(_colorsR)==_s.x()*_s.y() );
    tut::ensure( sizeof(_colorsG)==_s.x()*_s.y() );
    tut::ensure( sizeof(_colorsB)==_s.x()*_s.y() );
    unsigned char c=0;
    for(unsigned char i=0; i<sizeof(_colorsR); ++i)
    {
      _colorsR[i]=c++;
      _colorsG[i]=c++;
      _colorsB[i]=c++;
    }

    for(unsigned int y=0; y<_s.y(); ++y)
      for(unsigned int x=0; x<_s.x(); ++x)
      {
        unsigned char  pos=y*_s.x()+x;
        const PixelRGB p( _colorsR[pos],
                          _colorsG[pos],
                          _colorsB[pos] );
        _r.setPixel( Position(x,y), p );
      }
  }

  Size           _s;
  RasterRGB      _r;
  RasterLayerRGB _rl;
  unsigned char  _colorsR[2*3];
  unsigned char  _colorsG[2*3];
  unsigned char  _colorsB[2*3];
};

} // namespace Histogram
} // namespace Graphics


namespace tut
{
typedef Graphics::Histogram::GeneralTransformationTestClass TestClass;
typedef test_group<TestClass> factory;
typedef factory::object testObj;
} // namespace tut


namespace
{
tut::factory tf("Graphics/Histogram/GeneralTransformation");
}


using namespace boost;
using namespace Graphics;
using namespace Graphics::Histogram;
using namespace Graphics::Transform;

typedef General<PixelRGB, 256>               TGeneralTest;
typedef GeneralTransformation<PixelRGB, 256> TGeneralTransformationTest;

namespace tut
{

// test building of histogram via Transformation
// and check if it matches
template<>
template<>
void testObj::test<1>(void)
{
  TGeneralTest               h (_rl);
  TGeneralTransformationTest ht(_rl);

  try
  {
    ht.getHistogram();      // should throw or fail otherwise
    fail("data from uninitialized Transformation was recieved");
  }
  catch(const Exception &ex)
  {
    // this is expected behavior
  }

  Transformation     &t =ht;
  t.transform();            // perform histogram computation
  const TGeneralTest &rh=ht.getHistogram();

  // compare both histograms
  for(unsigned int i=0; i<256; ++i)
  {
    ensure( h[i]==rh[i] );
  }
}

} // namespace tut

