/*
 * General.hpp
 *
 * representation of generic mask for 2D image reaster.
 *
 */
#ifndef INCLUDE_GRAPHICS_MASK_GENERAL_HPP_FILE
#define INCLUDE_GRAPHICS_MASK_GENERAL_HPP_FILE

/* public header */

#include <vector>
#include <cassert>

#include "Graphics/Offset.hpp"


namespace Graphics
{
namespace Mask
{

//
// general mask - set of offsets with assigned
// used-defined values. it should be inheritaded
// from and filled with values before usage.
//
//   TUser - user defined type to be used as
//           second object in pair.
//
template<typename TUser>
class General
{
public:
  typedef typename std::pair<Offset, TUser>   TPair;
  typedef typename std::vector<TPair>         TContainer;
  typedef typename TContainer::const_iterator const_iterator;

  inline const_iterator begin(void) const
  {
    return _pair.begin();
  }
  inline const_iterator end(void) const
  {
    return _pair.end();
  }

protected:
  explicit General(size_t size=0)
  {
    _pair.reserve(size);
  }

  // add entry to offset set
  inline void add(const Offset &o, const TUser &u)
  {
    _pair.push_back( TPair(o,u) );
  }

private:
  TContainer _pair;
}; // class General

} // namespace Mask
} // namespace Graphics

#endif

