/*
 * Radial.hpp
 *
 * mask of radial shape, computed for given radius.
 *
 */
#ifndef INCLUDE_GRAPHICS_MASK_RADIAL_HPP_FILE
#define INCLUDE_GRAPHICS_MASK_RADIAL_HPP_FILE

/* public header */

#include "Graphics/Mask/General.hpp"


namespace Graphics
{
namespace Mask
{

//
// TUser         - type of user-defined value.
// TSetUserValue - function object generating user value
//                 for a given offset. use following form:
//      inline TUser operator()(const Offset &o) const;
//
template<typename TUser, typename TSetUserValue>
class Radial: public General<TUser>
{
public:
  // r - radius of mask to generate
  Radial(size_t r, const TSetUserValue &suv)
  {
    const int    rs=r;      // use signed value
    const double rr=r*r;    // remember sqare of r
    for(int y=-r; y<=rs; ++y)
      for(int x=-r; x<=rs; ++x)
      {
        if(x*x + y*y > rr)  // point outside of ring?
          continue;
        const Offset o(x,y);
        add( o, suv(o) );   // add next entry
      }
  }
}; // class Radial

} // namespace Mask
} // namespace Graphics

#endif

