/*
 * Radial.t.cpp
 *
 */
#include <tut.h>

#include "Graphics/Mask/Radial.hpp"


namespace
{
struct TestFunObj
{
  inline double operator()(const Graphics::Offset &o) const
  {
    return o.getX()+o.getY();
  }
}; // struct TestFuncObj
} // unnamed namespace


namespace Graphics
{
namespace Mask
{

struct RadialTestClass
{
  typedef Radial<double, TestFunObj> TestRad;
  RadialTestClass(void):
    _tfo()
  {
  }

  void testR(const TestRad &r,       const Offset *o,
                   bool    *present, const int     elems)
  {
    // init table with values
    for(int i=0; i<elems; ++i)
      present[i]=false;

    // test if all of radious are present
    for(int i=0; i<elems; ++i)
    {
      for(TestRad::const_iterator it=r.begin();
          it!=r.end(); ++it)
      {
        tut::ensure("too many elemenst in offsets set", i<elems);
        // check for matching offset
        if( (*it).first.getX()==o[i].getX() &&
            (*it).first.getY()==o[i].getY()    )
        {
          present[i]=true;
          tut::ensure( "offset value does not match expected",
                       o[i].getX()+o[i].getY()==(*it).second  );
          break;
        }
      }
    }
    for(int i=0; i<elems; ++i)
      tut::ensure("some element is not present", present[i] );
  }

  const TestFunObj _tfo;
};

} // namespace Mask
} // namespace Graphics


namespace tut
{
typedef Graphics::Mask::RadialTestClass TestClass;
typedef test_group<TestClass> factory;
typedef factory::object testObj;
} // namespace tut


namespace
{
tut::factory tf("Graphics/Mask/Radial");
}


using namespace Graphics;
using namespace Graphics::Mask;

namespace tut
{

// test construction for r=0
template<>
template<>
void testObj::test<1>(void)
{
  TestRad r(0, _tfo);
  ensure( ++r.begin()==r.end() );
  ensure( (*r.begin()).second==0 );
}

// test for r=1
template<>
template<>
void testObj::test<2>(void)
{
  TestRad r(1, _tfo);
  Offset o[5]={
                Offset( 0,-1),
                Offset( 0, 1),
                Offset(-1, 0),
                Offset( 1, 0),
                Offset( 0, 0)
              };
  bool present[5];

  testR(r, o, present, 5);
}

// test for r=2
template<>
template<>
void testObj::test<3>(void)
{
  TestRad r(2, _tfo);
  Offset o[9+4]={
                  Offset( 0,-2),

                  Offset(-1,-1),
                  Offset( 0,-1),
                  Offset( 1,-1),

                  Offset(-2, 0),
                  Offset(-1, 0),
                  Offset( 0, 0),
                  Offset( 1, 0),
                  Offset( 2, 0),

                  Offset(-1, 1),
                  Offset( 0, 1),
                  Offset( 1, 1),

                  Offset( 0, 2)
                };
  bool present[9+4];

  testR(r, o, present, 9+4);
}

} // namespace tut

