/*
 * Pixel.hpp
 *
 */
#ifndef INCLUDE_GRAPHICS_PIXEL_HPP_FILE
#define INCLUDE_GRAPHICS_PIXEL_HPP_FILE

/* public header */

#include <assert.h>

#include "System/Enum.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{

// we use it as a single color
typedef unsigned char color;


//
//
// basic beclarations used by later classes.
//
//


//
// base class for all pixels
//
template<typename TColor, int Dims>
class PixelBase
{
public:
  typedef TColor ColorType; // color type we use
  enum { dims=Dims };       // numer of dimmenctions (compile time const)

  inline const TColor &layer(int pos) const
  {
    assert( 0<=pos && pos<dims );
    return _l[pos];
  }
  inline void layer(int pos, TColor c)
  {
    assert( 0<=pos && pos<dims );
    _l[pos]=c;
  }

protected:
  // this class is not ment to be created directly
  inline PixelBase(void)
  {
  }

private:
  TColor _l[Dims];
}; // class PixelBase


//
// class represening single-valued pixels
//
template<typename TColor>
class PixelLayer: public PixelBase<TColor, 1>
{
private:
  typedef PixelBase<TColor, 1> TBase;
public:
  inline PixelLayer(void)
  {
    // after this call pixel has a random value
  }
  explicit inline PixelLayer(TColor c)
  {
    set(c);
  }

  inline TColor get(void) const
  {
    return TBase::layer(0);
  }
  inline void set(TColor c)
  {
    TBase::layer(0, c);
  }
}; // class PixelLayer



//
//
// below starts declarations of specific pixel
// types to be used with Raster class.
//
//


//
// class represening binary values
//
class PixelBinary: public PixelLayer<bool>
{
public:
  inline PixelBinary(void)
  {
    // after this call pixel has a random value
  }
  explicit inline PixelBinary(bool c):
    PixelLayer<bool>(c)
  {
  }
}; // class PixelBinary


//
// class representing single grayscale pixel
//
class PixelMono: public PixelLayer<color>
{
public:
  inline PixelMono(void)
  {
    // after this call pixel has random color
  }
  explicit inline PixelMono(color c):
    PixelLayer<color>(c)
  {
  }
}; // class PixelMono


//
// class representing single RGB pixel
//
class PixelRGB: public PixelBase<color, 3>
{
public:
  struct PixelLayerBaseEnum
  {
    typedef enum
    {
      R=0,
      G=1,
      B=2
    } Type;
  }; // struct PixelLayerBaseEnum
  // interface enum for specifing layer to extract
  typedef System::Enum<PixelLayerBaseEnum> Layer;


  inline PixelRGB(void)
  {
    // after this call pixel has random color
  }
  explicit inline PixelRGB(color r, color g, color b)
  {
    setR(r);
    setG(g);
    setB(b);
  }

  inline color getR(void) const
  {
    return layer(Layer::R);
  }
  inline void setR(color c)
  {
    layer(Layer::R, c);
  }

  inline color getG(void) const
  {
    return layer(Layer::G);
  }
  inline void setG(color c)
  {
    layer(Layer::G, c);
  }

  inline color getB(void) const
  {
    return layer(Layer::B);
  }
  inline void setB(color c)
  {
    layer(Layer::B, c);
  }
}; // class PixelRGB

} // namespace Graphics

#endif

