/*
 * PixelAdder.hpp
 *
 */
#ifndef INCLUDE_GRAPHICS_PIXELTRANSFORM_PIXELADDER_HPP_FILE
#define INCLUDE_GRAPHICS_PIXELTRANSFORM_PIXELADDER_HPP_FILE

/* public header */

#include <math.h>

#include "Graphics/Pixel.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace PixelTransform
{

// template for computing sum of
// 2 pixels of a build-in pixel types
struct PixelAdder
{
  template<typename TPixel>
  inline TPixel operator()(const TPixel &p1, const TPixel &p2) const
  {
    TPixel out;         // create
    (*this)(out, p1,p2);// compute sum
    return out;         // and return result
  }

  // Gray pixel addition
  inline void operator()(PixelMono &out, const PixelMono &p1,
                                         const PixelMono &p2) const
  {
    out.set( std::min(p1.get()+p2.get(), 255) );
  }

  // RGB pixel addition
  inline void operator()(PixelRGB &out, const PixelRGB &p1,
                                        const PixelRGB &p2) const
  {
    out.setR( std::min(p1.getR()+p2.getR(), 255) );
    out.setG( std::min(p1.getG()+p2.getG(), 255) );
    out.setB( std::min(p1.getB()+p2.getB(), 255) );
  }

  // Binary pixel addition
  inline void operator()(PixelBinary &out, const PixelBinary &p1,
                                           const PixelBinary &p2) const
  {
    out.set( p1.get()||p2.get() );
  }
}; // struct PixelAdder

} // namespace PixelTransform
} // namespace Graphics

#endif

