/*
 * PixelConverter.hpp
 *
 */
#ifndef INCLUDE_GRAPHICS_PIXELTRANSFORM_PIXELCONVERTER_HPP_FILE
#define INCLUDE_GRAPHICS_PIXELTRANSFORM_PIXELCONVERTER_HPP_FILE

/* public header */

#include "Graphics/Pixel.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace PixelTransform
{

// template for convertion between build-in pixel types
struct PixelConverter
{
  template<typename TPixOut, typename TPixIn>
  inline TPixOut operator()(const TPixIn &in) const
  {
    TPixOut out;        // create
    (*this)(out, in);   // convert
    return out;         // and return
  }


  // Gray -> RGB
  inline void operator()(PixelRGB &out, const PixelMono &in) const
  {
    out.setR( in.get() );
    out.setG( in.get() );
    out.setB( in.get() );
  }
  // Gray -> Binary
  inline void operator()(PixelBinary &out, const PixelMono &in) const
  {
    out.set( (in.get()>=255/2) );
  }

  // RGB -> Gray
  inline void operator()(PixelMono &out, const PixelRGB &in) const
  {
    out.set( rgb2mono(in) );
  }
  // RGB -> Binary
  inline void operator()(PixelBinary &out, const PixelRGB &in) const
  {
    const color c=rgb2mono(in);
    out.set( (c>=255/2) );
  }

  // Binary -> Gray
  inline void operator()(PixelMono &out, const PixelBinary &in) const
  {
    out.set( (in.get())?255:0 );
  }
  // Binary -> RGB
  inline void operator()(PixelRGB &out, const PixelBinary &in) const
  {
    const color c=(in.get())?255:0;
    out.setR(c);
    out.setG(c);
    out.setB(c);
  }

protected:
  inline color rgb2mono(const PixelRGB &in) const
  {
    // 0.2125*R + 0.7154*G + 0.0721*B
    const double v=in.getR()*0.2125 +
                   in.getG()*0.7154 +
                   in.getB()*0.0721;
    return static_cast<unsigned char>(v);
  }
}; // struct PixelConverter

} // namespace PixelTransform
} // namespace Graphics

#endif

