/*
 * PixelConverter.t.cpp
 *
 */
#include <tut.h>
#include <math.h>
#include <string>

#include "Graphics/PixelTransform/PixelConverter.hpp"

namespace Graphics
{
namespace PixelTransform
{

struct PixelConverterTestClass: public PixelConverter
{
};

} // namespace PixelTransform
} // namespace Graphics


namespace tut
{
typedef Graphics::PixelTransform::PixelConverterTestClass TestClass;
typedef test_group<TestClass> factory;
typedef factory::object testObj;
} // namespace tut


namespace
{
tut::factory tf("Graphics/PixelTransform/PixelConverter");
const double eps=255*(5.0/100); // 5% error is acceptable
}


using namespace std;
using namespace Graphics;
using namespace Graphics::PixelTransform;

namespace tut
{

// test gray->rgb pixel
template<>
template<>
void testObj::test<1>(void)
{
  PixelMono pIn(123);
  PixelRGB  pOut=PixelConverter().operator()
                            <PixelRGB,PixelMono>(pIn);
  ensure( pOut.getR()==pIn.get() &&
          pOut.getG()==pIn.get() &&
          pOut.getB()==pIn.get()    );
}

// gray->binary
template<>
template<>
void testObj::test<2>(void)
{
  PixelMono pIn1(255);
  PixelMono pIn2(  0);

  ensure( (*this).operator()<PixelBinary,PixelMono>(pIn1).get()==true  );
  ensure( (*this).operator()<PixelBinary,PixelMono>(pIn2).get()==false );
}

// rgb->gray
template<>
template<>
void testObj::test<3>(void)
{
  PixelRGB  pIn(100, 150, 200);
  PixelMono pOut=PixelConverter().operator()
                            <PixelMono, PixelRGB>(pIn);
  ensure( fabs( pOut.get() - rgb2mono(pIn) )<eps );
}

// rgb->binary
template<>
template<>
void testObj::test<4>(void)
{
  PixelRGB pIn1(  1,  1,  1);
  PixelRGB pIn2(200,200,200);

  ensure( (*this).operator()<PixelBinary,PixelRGB>(pIn1).get()==false );
  ensure( (*this).operator()<PixelBinary,PixelRGB>(pIn2).get()==true  );
}

// binary->gray
template<>
template<>
void testObj::test<5>(void)
{
  PixelBinary pIn1(true );
  PixelBinary pIn2(false);

  ensure( (*this).operator()<PixelMono,PixelBinary>(pIn1).get()==255 );
  ensure( (*this).operator()<PixelMono,PixelBinary>(pIn2).get()==  0 );
}

// binary->rgb
template<>
template<>
void testObj::test<6>(void)
{
  PixelBinary pIn1(true );
  PixelBinary pIn2(false);

  const PixelRGB pOut1=(*this).operator()<PixelRGB,PixelBinary>(pIn1);
  const PixelRGB pOut2=(*this).operator()<PixelRGB,PixelBinary>(pIn2);
  ensure( pOut1.getR()==255 && pOut1.getG()==255 && pOut1.getB()==255 );
  ensure( pOut2.getR()==  0 && pOut2.getG()==  0 && pOut2.getB()==  0 );
}

} // namespace tut

