/*
 * PixelDiffer.hpp
 *
 */
#ifndef INCLUDE_GRAPHICS_PIXELTRANSFORM_PIXELDIFFER_HPP_FILE
#define INCLUDE_GRAPHICS_PIXELTRANSFORM_PIXELDIFFER_HPP_FILE

/* public header */

#include <math.h>

#include "Graphics/Pixel.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace PixelTransform
{

// template for computing difference between
// 2 pixels of a build-in pixel types
struct PixelDiffer
{
  template<typename TPixel>
  inline TPixel operator()(const TPixel &p1, const TPixel &p2) const
  {
    TPixel out;         // create
    (*this)(out, p1,p2);// compute difference
    return out;         // and return result
  }

  // Gray pixel difference
  inline void operator()(PixelMono &out, const PixelMono &p1,
                                         const PixelMono &p2) const
  {
    out.set( abs(p1.get()-p2.get()) );
  }

  // RGB pixel difference
  inline void operator()(PixelRGB &out, const PixelRGB &p1,
                                        const PixelRGB &p2) const
  {
    out.setR( abs(p1.getR()-p2.getR()) );
    out.setG( abs(p1.getG()-p2.getG()) );
    out.setB( abs(p1.getB()-p2.getB()) );
  }

  // Binary pixel difference
  inline void operator()(PixelBinary &out, const PixelBinary &p1,
                                           const PixelBinary &p2) const
  {
    out.set( (p1.get()!=p2.get()) );
  }
}; // struct PixelDiffer

} // namespace PixelTransform
} // namespace Graphics

#endif

