/*
 * Generic.t.cpp
 *
 */
#include <tut.h>

#include "Graphics/Pyramid/Generic.hpp"

namespace Graphics
{
namespace Pyramid
{

struct GenericTestClass: public Generic<PixelMono>
{
  GenericTestClass(size_t h=3, double scaleFactor=2):
    Generic<PixelMono>( Size(32,64), h, scaleFactor ),
    _size(32,64)
  {
  }

  virtual void compute(void)
  {
    for(size_t i=0; i<height(); ++i)
      markPixel( *get(i).get(), i );
  }

  void markPixel(TRaster &r, size_t h)
  {
    const Position pos(0,0);
    r.setPixel(pos, PixelMono(h) );
  }

  Size _size;
};

} // namepsace Pyramid
} // namespace Graphics


namespace tut
{
typedef Graphics::Pyramid::GenericTestClass TestClass;
typedef test_group<TestClass> factory;
typedef factory::object testObj;
} // namespace tut


namespace
{
tut::factory tf("Graphics/Pyramid/Generic");
}


using namespace Graphics;
using namespace Graphics::Pyramid;

namespace tut
{

// check if sizes on every level are fine
template<>
template<>
void testObj::test<1>(void)
{
  const Size l1(32,64);
  const Size l2(16,32);
  const Size l3( 8,16);
  ensure( get(0)->size()==l1 );
  ensure( get(1)->size()==l2 );
  ensure( get(2)->size()==l3 );
}

// test if getFirst returns what it should
template<>
template<>
void testObj::test<2>(void)
{
  ensure( get(0)==getFirst() );
}

// try computation (call must be virtual!)
template<>
template<>
void testObj::test<3>(void)
{
  Generic<PixelMono> *base=this;
  base->compute();

  // now computation() should set (0;0) pixels to
  // value being their level number.
  const Position pos(0,0);
  for(size_t i=0; i<height(); ++i)
  {
    const PixelMono &pix=get(i)->getPixel(pos);
    ensure(pix.get()==i);
  }
}

// test scaling for diffrent scale factor
template<>
template<>
void testObj::test<4>(void)
{
  GenericTestClass gtc(2,4);    // h=2, scale=1/4
  const Size l1(32,64);
  const Size l2( 8,16);
  ensure( gtc.get(0)->size()==l1 );
  ensure( gtc.get(1)->size()==l2 );
}

// test throw on too many levels for a given size
template<>
template<>
void testObj::test<5>(void)
{
  try
  {
    GenericTestClass gtc(20,4); // h=20, scale=1/4
    fail("construction on too many pyramids level "
         "didn't failed as it should");
  }
  catch(const Graphics::Exception &ex)
  {
    // this should be throw
  }
}

// test throw on too small height
template<>
template<>
void testObj::test<6>(void)
{
  try
  {
    GenericTestClass gtc(1);  // h=1
    fail("construction of too short pyramid (h=1) "
         "didn't failed as it should");
  }
  catch(const Graphics::Exception &ex)
  {
    // this should be throw
  }
}

// test throw on bad scale factor
template<>
template<>
void testObj::test<7>(void)
{
  try
  {
    GenericTestClass gtc(2, 1); // h=2, scale=1
    fail("construction of pyramid with bac scale factor (sf=1) "
         "didn't failed as it should");
  }
  catch(const Graphics::Exception &ex)
  {
    // this should be throw
  }
}

// check if height is ok
template<>
template<>
void testObj::test<8>(void)
{
  GenericTestClass gtc(2);
  ensure(gtc.height()==2);
}

// test throw on incorect get()
template<>
template<>
void testObj::test<9>(void)
{
  GenericTestClass gtc(2, 2);   // h=2, scale=2
  try
  {
    gtc.get(2);                 // should throw here
    fail("get() on index out of bound"
         "didn't failed as it should");
  }
  catch(const Graphics::Exception &ex)
  {
    // this should be throw
  }
}

// test replace() protected call
template<>
template<>
void testObj::test<10>(void)
{
  Generic<PixelMono> *base=this;
  base->compute();

  const size_t   h2use=1;
  const Position pos(0,0);
  SharedRasterMono srm( get(h2use)->size() );   // size should be ok.
  srm->setPixel( Position(0,0), PixelMono(66) );// prepare pixel
  replace(srm, h2use);          // replace itself

  // now computation() should set (0;0) pixels to
  // value being their level number.
  for(size_t i=0; i<height(); ++i)
  {
    const PixelMono &pix=get(i)->getPixel(pos);
    if(i==h2use)
      ensure(pix.get()==66);
    else
      ensure(pix.get()== i);
  }
}

// test replace() to throw on error
template<>
template<>
void testObj::test<11>(void)
{
  // too big value - should throw
  SharedRasterMono srm1( getFirst()->size() );   // size should be ok.
  try
  {
    replace(srm1, 10000);       // replace itself
    fail("replace() didn't throw on too big 'h'");
  }
  catch(const Graphics::Exception &ex)
  {
    // this is expected
  }

  // try insering image of bad size
  SharedRasterMono srm2( Size(2,1111) );    // size should be incorrect.
  try
  {
    replace(srm2, 0);           // replace itself
    fail("replace() didn't throw on bad-sized image");
  }
  catch(const Graphics::Exception &ex)
  {
    // this is expected
  }
}

} // namespace tut

