/*
 * Raster.t.cpp
 *
 */
#include <tut.h>

#include "Graphics/Raster.hpp"

namespace Graphics
{

struct RasterTestClass
{
  RasterTestClass():
    _s(2,3),
    _r(_s)
  {
    for(unsigned int y=0; y<_s.y(); ++y)
      for(unsigned int x=0; x<_s.x(); ++x)
        _r.setPixel( Position(x,y), PixelMono(100*x+y) );
  }

  Size       _s;
  RasterMono _r;
};

} // namespace Graphics


namespace tut
{
typedef Graphics::RasterTestClass TestClass;
typedef test_group<TestClass> factory;
typedef factory::object testObj;
} // namespace tut


namespace
{
tut::factory tf("Graphics/Raster");
}


using namespace Graphics;

namespace tut
{

// check if construction was ok
template<>
template<>
void testObj::test<1>(void)
{
  ensure( _s==_r.size() );
  for(unsigned int y=0; y<_s.y(); ++y)
    for(unsigned int x=0; x<_s.x(); ++x)
      ensure( _r.getPixel( Position(x,y) ).get()==PixelMono(100*x+y).get() );
}

// test get/set pixel
template<>
template<>
void testObj::test<2>(void)
{
  PixelMono p  (216);
  Position  pos(1,0);
  ensure( _r.getPixel(pos).get()!=p.get() );
  _r.setPixelSec(pos, p);
  ensure( _r.getPixel(pos).get()==p.get() );

  ensure( _r.getPixel(pos).get()==_r.getPixelSec(pos).get() );
  try
  {
    Position pOut(10,20);
    _r.getPixelSec(pos);
    fail("getPixelSec() didn't throw as supossed");
  }
  catch(...) { /* should throw */ }
}

// test inRange()
template<>
template<>
void testObj::test<3>(void)
{
  Position in (1,2);
  Position out(9,8);
  ensure(  _r.inRange(in ) );
  ensure( !_r.inRange(out) );
}

// test filling whole image with a color
template<>
template<>
void testObj::test<4>(void)
{
  PixelMono p(6);
  _r.fill(p);
  for(unsigned int y=0; y<_s.y(); ++y)
    for(unsigned int x=0; x<_s.x(); ++x)
      ensure( _r.getPixel( Position(x,y) ).get()==p.get() );
}

// test getting pixel out of bounds of image,
// with default value
template<>
template<>
void testObj::test<5>(void)
{
  const PixelMono pDef(66);

  // test for pixel in rage
  {
    const Position  pos(1,2);
    const PixelMono p1 =_r.getPixel(pos);
    const PixelMono p2 =_r.getPixel(pos, pDef);
    ensure(p1.get()==p2.get());     // pixel should be taken correctly
    ensure(p1.get()!=pDef.get());   // and default one should not be the same
  }

  // test for pixel out of rage
  {
    const Position  pos(10,20);
    const PixelMono p=_r.getPixel(pos, pDef);
    ensure(p.get()==pDef.get());    // default pixel should be returned here
  }
}

// test if constructor of any common instance can throw
template<>
template<>
void testObj::test<6>(void)
{
  const Size   s (12,16);
  RasterRGB    r1(s);
  RasterMono   r2(s);
  RasterBinary r3(s);
}

// try copy-construction and assignment
template<>
template<>
void testObj::test<7>(void)
{
  const Size s(1,2);
  RasterMono r1(s);
  RasterMono r2(r1);
  RasterMono r3(s);
  r3=RasterMono(r1);

  ensure( r1.getPixel( Position(0,0) ).get()==
          r2.getPixel( Position(0,0) ).get()   );
  ensure( r1.getPixel( Position(0,1) ).get()==
          r2.getPixel( Position(0,1) ).get()   );

  ensure( r1.getPixel( Position(0,0) ).get()==
          r3.getPixel( Position(0,0) ).get()   );
  ensure( r1.getPixel( Position(0,1) ).get()==
          r3.getPixel( Position(0,1) ).get()   );
}

} // namespace tut

