/*
 * RasterLayer.t.cpp
 *
 */
#include <tut.h>

#include "Graphics/RasterLayer.hpp"

namespace Graphics
{

struct RasterLayerTestClass
{
  RasterLayerTestClass(void):
    _s(4,3),
    _r(_s),
    _rl(&_r, PixelRGB::Layer::G)
  {
  }

  Size                  _s;
  RasterRGB             _r;
  RasterLayer<PixelRGB> _rl;
};

} // namespace Graphics


namespace tut
{
typedef Graphics::RasterLayerTestClass TestClass;
typedef test_group<TestClass> factory;
typedef factory::object testObj;
} // namespace tut


namespace
{
tut::factory tf("Graphics/RasterLayer");
}


using namespace Graphics;

namespace tut
{

// test constructors for RasterLayer
template<>
template<>
void testObj::test<1>(void)
{
  RasterLayer<PixelRGB> rl(&_r, 0);

  PixelRGB::Layer l( PixelRGB::Layer::B );
  RasterLayerRGB  rlRGB=makeRasterLayerRGB(&_r, l);
}

// test basic methods
template<>
template<>
void testObj::test<2>(void)
{
  ensure( _rl.size()==_r.size() );

  const Position pos1(0,0);
  ensure( _rl.inRange(pos1)==_r.inRange(pos1) );

  const Position pos2(10,10);
  ensure( _rl.inRange(pos2)==_r.inRange(pos2) );
}

// test get
template<>
template<>
void testObj::test<3>(void)
{
  const Position pos(1,2);
  const PixelRGB pRGB=_r.getPixel(pos);
  const color    col =_rl.getPixel(pos);
  ensure( col==pRGB.getG() );
}

// test getSec
template<>
template<>
void testObj::test<4>(void)
{
  const Position  pos(1,2);
  const PixelRGB &pRGB=_r.getPixelSec(pos);
  const color    &col =_rl.getPixelSec(pos);
  ensure( col==pRGB.getG() );

  try
  {
    const Position pErr(10,20);
    _r.getPixelSec(pErr);
    fail("getPixelSec() didn't throw on error");
  }
  catch(const Graphics::Exception &ex)
  {
    // this is expected
  }
}

// set get with default value
template<>
template<>
void testObj::test<5>(void)
{
  const Position  pos(1,2);
  const color     defC=_rl.getPixel(pos)+66; // might wrapp - doesn't matter
  const PixelRGB &pRGB=_r.getPixel(pos);
  const color    &col =_rl.getPixel(pos, defC);
  ensure( col==pRGB.getG() && col!=defC );

  const Position  pErr(100,200);
  const color    &col2=_rl.getPixel(pErr, defC);
  ensure( col2==defC );
}

// set pixel
template<>
template<>
void testObj::test<6>(void)
{
  const Position pos(1,0);
  const PixelRGB pix =_r.getPixel(pos); // old value
  const color    cNew=pix.getG()+11;
  _rl.setPixel(pos, cNew);
  const PixelRGB pix2=_r.getPixel(pos); // new value
  ensure( pix2.getR()==pix.getR() &&
          pix2.getB()==pix.getB() &&
          pix2.getG()==cNew          );
}

// set pixel secure
template<>
template<>
void testObj::test<7>(void)
{
  const Position pos(3,1);
  const PixelRGB pix =_r.getPixel(pos); // old value
  const color    cNew=pix.getG()+11;
  _rl.setPixelSec(pos, cNew);
  const PixelRGB pix2=_r.getPixel(pos); // new value
  ensure( pix2.getR()==pix.getR() &&
          pix2.getB()==pix.getB() &&
          pix2.getG()==cNew          );

  try
  {
    const Position pos(102,303);
    const color    c  (99);
    _rl.setPixelSec(pos, c);
    fail("setPixelSec() didn't throw on error");
  }
  catch(const Graphics::Exception &ex)
  {
    // this is expected
  }
}

// test rasterRGB layer
template<>
template<>
void testObj::test<8>(void)
{
  RasterLayerRGB rl=makeRasterLayerRGB(&_r, PixelRGB::Layer::B);
  const Position pos(1,0);
  const PixelRGB pix =_r.getPixel(pos); // old value
  const color    cNew=pix.getB()+11;
  rl.setPixel(pos, cNew);
  const PixelRGB pix2=_r.getPixel(pos); // new value
  ensure( pix2.getR()==pix.getR() &&
          pix2.getG()==pix.getG() &&
          pix2.getB()==cNew          );
}

// try filling image layer with single color
template<>
template<>
void testObj::test<9>(void)
{
  // fill with some star value
  const color c1=16;
  _rl.fill(c1);
  for(size_t y=0; y<_rl.size().y(); ++y)
    for(size_t x=0; x<_rl.size().x(); ++x)
      ensure( _rl.getPixel( Position(x,y) )==c1 );

  // now try different to check if it differs
  const color c2=61;
  _rl.fill(c2);
  for(size_t y=0; y<_rl.size().y(); ++y)
    for(size_t x=0; x<_rl.size().x(); ++x)
      ensure( _rl.getPixel( Position(x,y) )==c2 );
}

// try building RasterLayerMono
template<>
template<>
void testObj::test<10>(void)
{
  RasterMono      r (_s);
  RasterLayerMono rl=makeRasterLayerMono(&r);
}


// try building RasterLayerBinary
template<>
template<>
void testObj::test<11>(void)
{
  RasterBinary      r(_s);
  RasterLayerBinary rl=makeRasterLayerBinary(&r);
}

} // namespace tut

