/*
 * Resizer.hpp
 *
 * basic class for all resizers. all resizers must
 * derive from this one.
 *
 */
#ifndef INCLUDE_GRAPHICS_SCALE_RESIZER_HPP_FILE
#define INCLUDE_GRAPHICS_SCALE_RESIZER_HPP_FILE

/* public header */

#include "Graphics/Raster.hpp"
#include "Graphics/AutoRaster.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace Scale
{

template<typename TPixel>
class Resizer
{
public:
  typedef Raster<TPixel>     TRaster;
  typedef AutoRaster<TPixel> TAutoRaster;

  virtual ~Resizer(void)
  {
  }

  // scaling method returning new object as output.
  inline TAutoRaster resizeToNew(const Size    &size,
                                 const TRaster &rIn  )
  {
    TAutoRaster ar(size);
    resize( *ar.get(), rIn );
    return ar;
  }

  // generic scaling method - input image is scaled to
  // output image size.
  virtual void resize(      TRaster &rOut,
                      const TRaster &rIn  ) = 0;

protected:
  typedef unsigned int  u_int;
}; // class Resizer

} // namespace Scale
} // namespace Graphics

#endif

