/*
 * Simple.hpp
 *
 * function object for image rescaling
 *
 */
#ifndef INCLUDE_GRAPHICS_SCALE_SIMPLE_HPP_FILE
#define INCLUDE_GRAPHICS_SCALE_SIMPLE_HPP_FILE

/* public header */

#include "Graphics/Scale/Resizer.hpp"


namespace Graphics
{
namespace Scale
{

template<typename TPixel>
class Simple: public Resizer<TPixel>
{
public:
  typedef typename Resizer<TPixel>::TRaster TRaster;

  //
  // simple scalling method - operates on user
  // output data. scale is computed automaticaly
  // from image sizes difference.
  //
  virtual void resize(      TRaster &rOut,
                      const TRaster &rIn  )
  {
    const Size   outSize=rOut.size();
    const Size   inSize =rIn.size();
    const double xFact  =(1.0*inSize.x())/outSize.x();
    const double yFact  =(1.0*inSize.y())/outSize.y();
    // get proper pixels
    for(u_int y=0; y<outSize.y(); ++y)
      for(u_int x=0; x<outSize.x(); ++x)
      {
        // compute output position
        const u_int xIn=static_cast<u_int>(x*xFact);
        const u_int yIn=static_cast<u_int>(y*yFact);
        // assing pixel to output
        rOut.setPixel(    Position(x,  y  ),
            rIn.getPixel( Position(xIn,yIn) ) );
      }
  }
}; // class Simple

} // namespace Scale
} // namespace Graphics

#endif

