/*
 * Simple.t.cpp
 *
 */
#include <tut.h>

#include "Graphics/Scale/Simple.hpp"

namespace Graphics
{
namespace Scale
{

struct SimpleTestClass
{
  bool compare(const RasterMono &r1,
               const RasterMono &r2) const
  {
    if( r1.size()!=r2.size() )
      return false;

    const Size size=r1.size();
    for(unsigned int y=0; y<size.y(); ++y)
      for(unsigned int x=0; x<size.x(); ++x)
      {
        const Position p(x,y);
        if( r1.getPixel(p).get()!=r2.getPixel(p).get() )
          return false;
      }
    return true;
  }
};

} // namespace Scale
} // namespace Graphics


namespace tut
{
typedef Graphics::Scale::SimpleTestClass TestClass;
typedef test_group<TestClass> factory;
typedef factory::object testObj;
} // namespace tut


namespace
{
tut::factory tf("Graphics/Scale/Simple");
}


using namespace Graphics;
using namespace Graphics::Scale;

namespace tut
{

// try scalling 1:1
template<>
template<>
void testObj::test<1>(void)
{
  const Size size(2,2);
  RasterMono rIn (size);
  RasterMono rOut(size);
  Simple<PixelMono>().resize(rOut, rIn);
  ensure( compare( rIn, rOut ) );
}

// test real scalling
template<>
template<>
void testObj::test<2>(void)
{
  const Size sizeIn (4,4);
  const Size sizeOut(2,2);
  RasterMono rIn (sizeIn);
  RasterMono rOut(sizeOut);
  Simple<PixelMono>().resize(rOut, rIn);
  // check if it succeded (if proper pixel were choosen)
  // NOTE: this is not the only way to do the scalling,
  //       but it is most obvious, so it will be treated
  //       as final test anyway.
  for(unsigned int y=0; y<sizeOut.y(); ++y)
    for(unsigned int x=0; x<sizeOut.x(); ++x)
    {
      const unsigned int xIn=x*2;
      const unsigned int yIn=y*2;
      ensure( rOut.getPixel( Position(x,  y  ) ).get() ==
              rIn.getPixel ( Position(xIn,yIn) ).get()    );
    }
}

} // namespace tut

