/*
 * SimpleTransformation.hpp
 *
 * transformation of image to different scale
 *
 */
#ifndef INCLUDE_GRAPHICS_SCALE_SIMPLETRANSFORMATION_HPP_FILE
#define INCLUDE_GRAPHICS_SCALE_SIMPLETRANSFORMATION_HPP_FILE

/* public header */

#include "Graphics/SharedRaster.hpp"
#include "Graphics/Scale/Simple.hpp"
#include "Graphics/Transform/Transformation.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace Scale
{

template<typename TPixel>
class SimpleTransformation: public Graphics::Transform::Transformation
{
public:
  typedef SharedRaster<TPixel> TShrRaster;

  SimpleTransformation(TShrRaster rOut, TShrRaster rIn):
    _rIn (rIn ),
    _rOut(rOut)
  {
  }

  inline TShrRaster getRasterIn(void) const
  {
    return _rIn;
  }
  inline TShrRaster getRasterOut(void) const
  {
    return _rOut;
  }

  virtual void transform(void)
  {
    Simple<TPixel>().resize( *_rOut.get(), *_rIn.get() );
  }

private:
  TShrRaster _rIn;
  TShrRaster _rOut;
}; // class SimpleTransformation

} // namespace Scale
} // namespace Graphics

#endif

