/*
 * SharedRaster.hpp
 *
 * Raster class wrapped to be pointer-copyable.
 *
 */
#ifndef INCLUDE_GRAPHICS_SHAREDRASTER_HPP_FILE
#define INCLUDE_GRAPHICS_SHAREDRASTER_HPP_FILE

/* public header */

#include <boost/shared_ptr.hpp>
#include <cassert>

#include "Graphics/Raster.hpp"


namespace Graphics
{

template<typename TPixel>
class SharedRaster: public boost::shared_ptr< Raster<TPixel> >
{
public:
  typedef Raster<TPixel> TRaster;

  // construct Raster of a given size
  explicit SharedRaster(const Graphics::Size &size):
    boost::shared_ptr< Raster<TPixel> >( new TRaster(size) )
  {
  }
  // make object directly from object
  explicit SharedRaster(TRaster *r):
    boost::shared_ptr< Raster<TPixel> >(r)
  {
    assert(r!=NULL);
  }
}; // class SharedRaster

//
// here are some commonly used definitions
//
typedef SharedRaster<PixelRGB>    SharedRasterRGB;
typedef SharedRaster<PixelMono>   SharedRasterMono;
typedef SharedRaster<PixelBinary> SharedRasterBinary;

} // namespace Graphics

#endif

