/*
 * SharedSyncRasterLayer.hpp
 *
 * thread safe version of RasterLayer class - copyable version.
 *
 */
#ifndef INCLUDE_GRAPHICS_SHAREDSYNCRASTERLAYER_HPP_FILE
#define INCLUDE_GRAPHICS_SHAREDSYNCRASTERLAYER_HPP_FILE

/* public header */

#include "System/Threads/SharedSyncObj.hpp"
#include "Graphics/SyncRasterLayer.hpp"


namespace Graphics
{

template<typename TPixel>
class SharedSyncRasterLayer: public
            System::Threads::SharedSyncObj< SyncRasterLayer<TPixel> >
{
public:
  typedef Raster<TPixel>          TRaster;
  typedef SyncRasterLayer<TPixel> TSyncRasterLayer;
  typedef System::Threads::SharedSyncObj< SyncRasterLayer<TPixel> > TBase;

  explicit SharedSyncRasterLayer(TRaster *r, size_t layerNo):
    TBase( new TSyncRasterLayer(r, layerNo) )
  {
  }

private:
}; // class SharedSyncRasterLayer

//
// here are some commonly used definitions
//
typedef SharedSyncRasterLayer<PixelRGB>    SharedSyncRasterLayerRGB;
typedef SharedSyncRasterLayer<PixelMono>   SharedSyncRasterLayerMono;
typedef SharedSyncRasterLayer<PixelBinary> SharedSyncRasterLayerBinary;

} // namespace Graphics

#endif

