/*
 * Storer.hpp
 *
 * interface for reading/writing images to a given
 * file descriptor (does not has to be disk file!).
 *
 */
#ifndef INCLUDE_GRAPHICS_STORAGE_STORER_HPP_FILE
#define INCLUDE_GRAPHICS_STORAGE_STORER_HPP_FILE

/* public header */

#include <string>
#include <boost/noncopyable.hpp>

#include "Graphics/Exception.hpp"

namespace Graphics
{
namespace Storage
{

//
// general storage class - represents pair
// of storage image and storage descriptor.
// storage file is given by its disk name -
// the data to store should be passed from
// constructor of deriving class, and used during
// read()/write() virtual methods call.
//
class Storer: private boost::noncopyable
{
public:
  // fileName - name of file to write image to / read image from
  Storer(const std::string &fileName):
    _fileName(fileName)
  {
  }
  virtual ~Storer(void)
  {
  }

  //
  // I/O for a given descriptor - user implemented
  //
  virtual void read(void)=0;
  virtual void write(void)=0;

protected:
  inline const std::string &getFileName(void) const
  {
    return _fileName;
  }

private:
  const std::string _fileName;
}; // class Storer

} // namespace Storage
} // namespace Graphics

#endif

