/*
 * StorerByExt.hpp
 *
 */
#ifndef INCLUDE_GRAPHICS_STORAGE_STORERBYEXT_HPP_FILE
#define INCLUDE_GRAPHICS_STORAGE_STORERBYEXT_HPP_FILE

/* public header */

#include <boost/shared_ptr.hpp>

#include "Graphics/Raster.hpp"
#include "Graphics/Storage/Storer.hpp"
#include "Graphics/Exception.hpp"

namespace Graphics
{
namespace Storage
{

class StorerByExt: public Storer
{
public:
  StorerByExt(const std::string &fileName,
              boost::shared_ptr<RasterRGB> raster):
    Storer(fileName),
    _raster(raster)
  {
  }

  // note: read() might change image size!
  virtual void read(void);
  virtual void write(void);

private:
  boost::shared_ptr<RasterRGB> _raster;
}; // class StorerByExt

//
// definitions of storers that use this one
//

typedef StorerByExt StorerBMP;
typedef StorerByExt StorerPNG;
typedef StorerByExt StorerGIF;
typedef StorerByExt StorerJPG;
typedef StorerByExt StorerTIF;

} // namespace Storage
} // namespace Graphics

#endif

