/*
 * SyncRasterLayer.hpp
 *
 * thread safe version of RasterLayer class.
 *
 */
#ifndef INCLUDE_GRAPHICS_SYNCRASTERLAYER_HPP_FILE
#define INCLUDE_GRAPHICS_SYNCRASTERLAYER_HPP_FILE

/* public header */

#include "System/Threads/SyncObj.hpp"
#include "Graphics/RasterLayer.hpp"


namespace Graphics
{

template<typename TPixel>
class SyncRasterLayer: public
            System::Threads::SyncObj< RasterLayer<TPixel> >
{
public:
  typedef Raster<TPixel> TRaster;

  explicit SyncRasterLayer(TRaster *r, size_t layerNo):
    _rl(r, layerNo)
  {
    // initialize base class with proper Raster pointer
    init(&_rl);
  }

private:
  RasterLayer<TPixel> _rl;
}; // class SyncRasterLayer

//
// here are some commonly used definitions
//
typedef Graphics::SyncRasterLayer<Graphics::PixelRGB>    SyncRasterLayerRGB;
typedef Graphics::SyncRasterLayer<Graphics::PixelMono>   SyncRasterLayerMono;
typedef Graphics::SyncRasterLayer<Graphics::PixelBinary> SyncRasterLayerBinary;

} // namespace Graphics

#endif

