/*
 * Transformation.hpp
 *
 * abstract for image transformation action.
 * it is sugested to use SharedRaster<> image
 * representation as input/output for transformations.
 *
 */
#ifndef INCLUDE_GRAPHICS_TRANSFORM_TRANSFORMATION_HPP_FILE
#define INCLUDE_GRAPHICS_TRANSFORM_TRANSFORMATION_HPP_FILE

/* public header */

#include <boost/noncopyable.hpp>

#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace Transform
{

//
// class for abstracting image transformation operations.
// derived class should already set it's input/output
// data in constructor, so that separate call on such an
// image object will perform all necessary operations.
//
class Transformation: public boost::noncopyable
{
public:
  virtual ~Transformation(void)
  {
  }

  // call to be user-implemented operation
  virtual void transform(void)=0;
}; // class Transformation

} // namespace Transform
} // namespace Graphics

#endif

