/*
 * CaptureConfigurator.hpp
 *
 * common interface for video frame capturing regardles
 * of API1 or API2 interface usage.
 *
 */
#ifndef INCLUDE_GRAPHICS_V4L_CAPTURECONFIGURATOR_HPP_FILE
#define INCLUDE_GRAPHICS_V4L_CAPTURECONFIGURATOR_HPP_FILE

/* public header */

#include <boost/shared_ptr.hpp>
#include <boost/noncopyable.hpp>

#include "Graphics/Size.hpp"
#include "Graphics/V4L/Device.hpp"
#include "Graphics/V4L/Descriptors.hpp"
#include "Graphics/V4L/Translations.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace V4L
{

class CaptureConfigurator: private boost::noncopyable
{
public:
  CaptureConfigurator( boost::shared_ptr<Device> dev):
    _dev(dev)
  {
  }
  virtual ~CaptureConfigurator(void)
  {
  }

  // configure using V4L[12] specific API. gets size to which
  // configure the device, and return's size that driver
  // accepted (they may differ!).
  virtual Size configure(const Size &newSize, ColorPalette cp)=0;

protected:
  Device& getDevice(void)
  {
    return *_dev.get();
  }

private:
  boost::shared_ptr<Device> _dev;
}; // class CaptureConfigurator

} // namespace V4L
} // namespace Graphics

#endif

