/*
 * Capturer.mt.cpp
 *
 */
#include <iostream>

#include "Graphics/Raster.hpp"
#include "Graphics/SharedRaster.hpp"
#include "Graphics/Storage/Storers.hpp"
#include "Graphics/V4L/Capturer.hpp"

using namespace std;
using namespace boost;
using namespace Graphics;
using namespace Graphics::V4L;
using namespace Graphics::Storage;



//
// main()
//
int main(int argc, char **argv)
{
  try
  {
    if(argc!=2)
    {
      cerr<<argv[0]<<" <output.png>"<<endl;
      return 1;
    }

    shared_ptr<Device> dev( new Device("/dev/video0",
                                       APIType::AUTO) );
    Capturer           cap( dev, Size(320,240),
                            ColorPalette(ColorPalette::RGB24) );

    // try capture few frames - device may need time it init...
    AutoRasterRGB ar(NULL);         // make output holder
    for(int i=0; i<6 && ar.get()==NULL; ++i)
      ar=cap.capture<PixelRGB>();   // capture single frame
    if( ar.get()==NULL )            // no frame captured?
    {
      cerr<<argv[0]<<": no frames were captured :(\n";
      return 4;
    }

    // now save image to disk
    SharedRasterRGB shr( ar.release() );            // switch pointer type
    StorerPNG       png( string(argv[1]), shr );    // create image storer
    png.write();                    // write data to disk
  }
  catch(const Graphics::V4L::Capturer::Exception &ex)
  {
    cerr<<argv[0]<<": Graphics::V4L::Capturer::Exception: "<<ex.what()<<endl;
    return 200;
  }
  catch(const Graphics::Exception &ex)
  {
    cerr<<argv[0]<<": Graphics::Exception: "<<ex.what()<<endl;
    return 200;
  }
  catch(const System::Exception &ex)
  {
    cerr<<argv[0]<<": System::Exception: "<<ex.what()<<endl;
    return 201;
  }
  catch(...)
  {
    cerr<<argv[0]<<": unkown exception cought\n";
    return 202;
  }

  return 0;
} // main()

