/*
 * Capturer.t.cpp
 *
 */
#include <tut.h>

#include "Graphics/Raster.hpp"
#include "Graphics/V4L/Capturer.hpp"

namespace Graphics
{
namespace V4L
{

struct CapturerTestClass
{
  CapturerTestClass(void):
    _devName("/dev/video0"),
    _cp     (ColorPalette::RGB24),
    _size   (320, 240),
    _dev    ( new Device(_devName, APIType::AUTO) ),
    _cap    (_dev, _size, _cp)
  {
  }

  std::string               _devName;
  ColorPalette              _cp;
  Size                      _size;
  boost::shared_ptr<Device> _dev;
  Capturer                  _cap;
};

} // namespace V4L
} // namespace Graphics


namespace tut
{
typedef Graphics::V4L::CapturerTestClass TestClass;
typedef test_group<TestClass> factory;
typedef factory::object testObj;
} // namespace tut


namespace
{
tut::factory tf("Graphics/V4L/Capturer");
}


using namespace Graphics;
using namespace Graphics::V4L;

namespace tut
{

// test constructor (reinitialization of device)
template<>
template<>
void testObj::test<1>(void)
{
  for(int i=0; i<2; ++i)
    Capturer c(_dev, _size, _cp);
}

// try capturing few frames
template<>
template<>
void testObj::test<2>(void)
{
  _size=_cap.size();            // check the size set to driver

  // try capturing few frames - at least one should pass!
  bool hasOne=false;
  for(int i=0; i<6 && !hasOne; ++i)
  {
    AutoRasterRGB ar=_cap.capture<PixelRGB>();  // capture single frame
    if( ar.get()!=NULL )        // succes?
      hasOne=true;
  }
  ensure("no frame has been grabbed", hasOne);
}

// ensure that errors will be reported,
// when wrong raster will be choosen
template<>
template<>
void testObj::test<3>(void)
{
  _size=_cap.size();            // check the size set to driver
  try
  {
    _cap.capture<PixelMono>();
    fail("calling capture with wrong raster type didn't throw");
  }
  catch(const Graphics::V4L::Capturer::Exception &ex)
  {
    // this call should throw
  }
}

} // namespace tut

