/*
 * Device.cpp
 *
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <errno.h>

#include "System/nonint_ioctl.hpp"
#include "Graphics/V4L/API1/APICheck.hpp"
#include "Graphics/V4L/API2/APICheck.hpp"
#include "Graphics/V4L/Device.hpp"

using namespace std;
using namespace System;

namespace Graphics
{
namespace V4L
{

Device::Device(const string  &devFile,
               const APIType  apiType):
  _dev( devFile, O_RDWR ),  // open device
  _api(apiType)             // temporary - will be overwritten by
                            // checkForAPI() call
{
  _api=checkForAPI(_api);   // choose proper API
}


APIType Device::checkForAPI(APIType assumed)
{
  switch( assumed.toInt() )
  {
    // automatic: try V4L2, if not proper fall back to V4L1
    case APIType::AUTO:
      if( API2::isAPI2Device( _dev.get() ) )
        return APIType::V4L2;
      else
        if( API1::isAPI1Device( _dev.get() ) )
          return APIType::V4L1;
      break;

    // test if it can be used as V4L1 device
    case APIType::V4L1:
      if( API1::isAPI1Device( _dev.get() ) )
        return APIType::V4L1;
      break;

    // test if this is V4L2 device
    case APIType::V4L2:
      if( API2::isAPI2Device( _dev.get() ) )
        return APIType::V4L2;
      break;
  } // switch(assumedAPIType)
  // fall back:
  throw Exception("Device::checkForAPI(): "
                  "requested API not supported by given device");
}


int Device::ioctl(int req, void *data)
{
  return nonint_ioctl( _dev.get(), req, data );
}

} // namespace V4L
} // namespace Graphics

