/*
 * Device.hpp
 *
 * V4L device representation wrapper.
 *
 */
#ifndef INCLUDE_GRAPHICS_V4L_DEVICE_HPP_FILE
#define INCLUDE_GRAPHICS_V4L_DEVICE_HPP_FILE

/* public header */

#include <string>
#include <boost/noncopyable.hpp>
#include <cassert>

#include "System/DiskFile.hpp"
#include "Graphics/V4L/Descriptors.hpp"
#include "Graphics/Exception.hpp"


namespace Graphics
{
namespace V4L
{

class Device: private boost::noncopyable
{
public:
  Device(const std::string& devFile,
         const APIType      apiType = APIType::AUTO);

  //
  // return descriptor of v4l file device
  // note: this is R/W-only copy! do NOT close(),
  //       nor use after destroing of Device class!
  //
  inline int getDev(void)
  {
    return _dev.get();
  }
  // return other parameters:
  inline const std::string& getDevFile(void) const
  {
    return _dev.getName();
  }
  inline APIType getAPIType(void) const
  {
    assert( _api!=APIType::AUTO );  // at this point proper api should be set
    return _api;
  }

  // frontend to calling ioctl()'s on this device
  int ioctl(int req, void *data);

private:
  // by assumed API type call checks if it is possible
  // to use given device with this one
  APIType checkForAPI(APIType assumed);

  System::DiskFile _dev;
  APIType          _api;
}; // class Device

} // namespace V4L
} // namespace Graphics

#endif

